@@ -26,10 +26,11 @@ const DEFAULT_LOCAL_ADDR: (Ipv4Addr, u16) = (Ipv4Addr::UNSPECIFIED, 0);
 pub const DEFAULT_QUERY_TIMEOUT: Duration = Duration::from_secs(90);
 
 const DEFAULT_ROOT_DIR_NAME: &str = "root_dir";
+const SN_CLIENT_QUERY_TIMEOUT: &str = "SN_QUERY_TIMEOUT";
 
 /// Configuration for sn_client.
 #[derive(Clone, Debug, Deserialize, Serialize)]
-pub struct Config {
+pub struct ClientConfig {
     /// The local address to bind to.
     pub local_addr: SocketAddr,
     /// Path to local storage.
@@ -44,7 +45,7 @@ pub struct Config {
     pub standard_wait: Duration,
 }
 
-impl Config {
+impl ClientConfig {
     /// Returns a new `Config` instance.
     ///
     /// This will try to read QuicP2P configuration from `config_file_path`, or else use the default
@@ -73,10 +74,28 @@ impl Config {
             Some(path) => read_config_file(path).await.unwrap_or_default(),
         };
 
+        qp2p.idle_timeout = Some(DEFAULT_QUERY_TIMEOUT);
+        qp2p.keep_alive_interval = Some(Duration::from_secs(30));
+
         let query_timeout = query_timeout.unwrap_or(DEFAULT_QUERY_TIMEOUT);
 
-        qp2p.idle_timeout = Some(Duration::from_secs(5));
-        qp2p.keep_alive_interval = Some(Duration::from_secs(1));
+        // if we have an env var for this, lets override
+        let query_timeout = match std::env::var(SN_CLIENT_QUERY_TIMEOUT) {
+            Ok(timeout) => match timeout.parse() {
+                Ok(time) => {
+                    warn!(
+                        "Query timeout set from env var {:?}",
+                        SN_CLIENT_QUERY_TIMEOUT
+                    );
+                    Duration::from_secs(time)
+                }
+                Err(error) => {
+                    warn!("There was an error parsing {:?} env var. Default or client configured query timeout will be used: {:?}", SN_CLIENT_QUERY_TIMEOUT, error);
+                    query_timeout
+                }
+            },
+            Err(_) => query_timeout,
+        };
 
         Self {
             local_addr: local_addr.unwrap_or_else(|| SocketAddr::from(DEFAULT_LOCAL_ADDR)),
@@ -94,7 +113,7 @@ async fn read_config_file(filepath: &Path) -> Result<QuicP2pConfig, Error> {
     let mut file = File::open(filepath).await?;
 
     let mut contents = vec![];
-    let _ = file.read_to_end(&mut contents).await?;
+    let _size = file.read_to_end(&mut contents).await?;
 
     serde_json::from_slice(&contents).map_err(|err| {
         warn!(
@@ -154,7 +173,7 @@ mod tests {
 
         // In the absence of a config file, the config handler
         // should initialize bootstrap_cache_dir only
-        let config = Config::new(
+        let config = ClientConfig::new(
             Some(&root_dir),
             None,
             genesis_key,
@@ -170,28 +189,33 @@ mod tests {
             .to_string();
         // normalise for mac
         if str_path.ends_with('/') {
-            let _ = str_path.pop();
+            let _some_last_char = str_path.pop();
         }
 
-        let expected_config = Config {
+        let expected_config = ClientConfig {
             local_addr: (Ipv4Addr::UNSPECIFIED, 0).into(),
             root_dir: root_dir.clone(),
             genesis_key,
-            qp2p: QuicP2pConfig::default(),
+            qp2p: QuicP2pConfig {
+                idle_timeout: Some(Duration::from_secs(90)),
+                keep_alive_interval: Some(Duration::from_secs(30)),
+                ..Default::default()
+            },
             query_timeout: DEFAULT_QUERY_TIMEOUT,
             standard_wait: DEFAULT_QUERY_TIMEOUT / 10,
         };
+        assert_eq!(format!("{:?}", config), format!("{:?}", expected_config));
         assert_eq!(serialize(&config)?, serialize(&expected_config)?);
 
         create_dir_all(&root_dir).await?;
         let mut file = File::create(&config_filepath)?;
 
         let config_on_disk =
-            Config::new(None, None, genesis_key, Some(&config_filepath), None, None).await;
+            ClientConfig::new(None, None, genesis_key, Some(&config_filepath), None, None).await;
         serde_json::to_writer_pretty(&mut file, &config_on_disk)?;
         file.sync_all()?;
 
-        let read_cfg = Config::new(None, None, genesis_key, None, None, None).await;
+        let read_cfg = ClientConfig::new(None, None, genesis_key, None, None, None).await;
         assert_eq!(serialize(&config_on_disk)?, serialize(&read_cfg)?);
 
         Ok(())
