@@ -67,6 +67,31 @@ use tracing_subscriber::{
     registry::LookupSpan,
 };
 
+/// Number of elders per section.
+pub(crate) const DEFAULT_ELDER_COUNT: usize = 7;
+const SN_ELDER_COUNT: &str = "SN_ELDER_COUNT";
+
+/// Get the expected elder count for our network, defaults to DEFAULT_ELDER_COUNT, but can be overridden by the env var SN_ELDER_COUNT
+pub(crate) fn elder_count() -> usize {
+    // if we have an env var for this, lets override
+    match std::env::var(SN_ELDER_COUNT) {
+        Ok(count) => match count.parse() {
+            Ok(count) => {
+                warn!(
+                    "ELDER_COUNT countout set from env var SN_ELDER_COUNT: {:?}",
+                    SN_ELDER_COUNT
+                );
+                count
+            }
+            Err(error) => {
+                warn!("There was an error parsing {:?} env var. Defaultelder_count() will be used: {:?}", SN_ELDER_COUNT, error);
+                DEFAULT_ELDER_COUNT
+            }
+        },
+        Err(_) => DEFAULT_ELDER_COUNT,
+    }
+}
+
 #[cfg(test)]
 use std::sync::Once;
 
@@ -159,12 +184,13 @@ pub fn init_test_logger() {
             .with_env_filter(tracing_subscriber::EnvFilter::from_default_env())
             .with_target(false)
             .event_format(LogFormatter::default())
-            .init()
+            .try_init().unwrap_or_else(|_| println!("Error initializing logger"));
     });
 }
 
 #[cfg(test)]
 mod tests {
+    use crate::elder_count;
     use crate::routing::log_markers::LogMarker;
     use crate::testnet_grep::search_testnet_results_per_node;
     use eyre::Result;
@@ -174,17 +200,23 @@ mod tests {
     #[tokio::test(flavor = "multi_thread")]
     #[ignore = "Testnet network_assert_ tests should be excluded from normal tests runs, they need to be run in sequence to ensure validity of checks"]
     async fn split_network_assert_health_check() -> Result<()> {
-        let prefix1_prior_elder_nodes =
-            search_testnet_results_per_node(format!(r"{}: Prefix\(1\)", LogMarker::SplitSuccess))?
-                .len();
+        let promoted_to_elder_nodes =
+            search_testnet_results_per_node(LogMarker::PromotedToElder.to_string())?.len();
+        let prefix1_prior_elder_nodes = search_testnet_results_per_node(format!(
+            r"{}: Prefix\(1\)",
+            LogMarker::StillElderAfterAplit
+        ))?
+        .len();
         let prefix1_new_elder_nodes = search_testnet_results_per_node(format!(
             r"{}: Prefix\(1\)",
             LogMarker::PromotedToElder
         ))?
         .len();
-        let prefix0_prior_elder_nodes =
-            search_testnet_results_per_node(format!(r"{}: Prefix\(0\)", LogMarker::SplitSuccess))?
-                .len();
+        let prefix0_prior_elder_nodes = search_testnet_results_per_node(format!(
+            r"{}: Prefix\(0\)",
+            LogMarker::StillElderAfterAplit
+        ))?
+        .len();
         let prefix0_new_elder_nodes = search_testnet_results_per_node(format!(
             r"{}: Prefix\(0\)",
             LogMarker::PromotedToElder
@@ -195,6 +227,7 @@ mod tests {
             search_testnet_results_per_node(LogMarker::SplitSuccess.to_string())?.len();
 
         println!("Found splits: {:?}", split_count);
+        println!("Promoted to elder so far: {:?}", promoted_to_elder_nodes);
 
         let total_elders = prefix0_prior_elder_nodes
             + prefix0_new_elder_nodes
@@ -214,13 +247,13 @@ mod tests {
         );
         println!("Found prefix_1_new_elders: {:?}", prefix1_new_elder_nodes);
 
-        assert!(prefix0_new_elder_nodes + prefix0_prior_elder_nodes >= 7);
-        assert!(prefix1_prior_elder_nodes + prefix1_new_elder_nodes >= 7);
+        assert!(prefix0_new_elder_nodes + prefix0_prior_elder_nodes >= elder_count());
+        assert!(prefix1_prior_elder_nodes + prefix1_new_elder_nodes >= elder_count());
 
         // we're not discounting demotions at the moment, so just more than 14 is fine
-        assert!(total_elders >= 14);
+        assert!(total_elders >= 2 * elder_count());
 
-        assert!(split_count >= 7);
+        assert!(split_count >= elder_count());
 
         Ok(())
     }
