@@ -8,8 +8,9 @@
 
 use super::{delivery_group, Comm, Core};
 use crate::dbs::UsedSpace;
-use crate::messaging::{system::Proposal, WireMsg};
+use crate::messaging::WireMsg;
 use crate::routing::{
+    core::Proposal,
     error::Result,
     log_markers::LogMarker,
     network_knowledge::{NetworkKnowledge, NodeState, SectionAuthorityProvider, SectionKeyShare},
@@ -72,7 +73,7 @@ impl Core {
     }
 
     pub(crate) async fn section_chain(&self) -> SecuredLinkedList {
-        self.network_knowledge.chain().await
+        self.network_knowledge.section_chain().await
     }
 
     /// Is this node an elder?
@@ -105,16 +106,12 @@ impl Core {
         Ok(self.key_share().await?.public_key_set)
     }
 
-    /// Returns the info about the section matching the name.
+    /// Returns the SAP of the section matching the name.
     pub(crate) async fn matching_section(
         &self,
         name: &XorName,
     ) -> Result<SectionAuthorityProvider> {
-        if self.network_knowledge.prefix().await.matches(name) {
-            Ok(self.network_knowledge.authority_provider().await)
-        } else {
-            self.network_knowledge.section_by_name(name)
-        }
+        self.network_knowledge.section_by_name(name)
     }
 
     /// Returns our index in the current BLS group if this node is a member of one, or
@@ -219,11 +216,12 @@ impl Core {
         excluded_names: &BTreeSet<XorName>,
     ) -> Result<Vec<Command>> {
         let mut commands = vec![];
+        let our_name = self.node.read().await.name();
 
         debug!("{}", LogMarker::TriggeringPromotionAndDemotion);
         for elder_candidates in self
             .network_knowledge
-            .promote_and_demote_elders(&self.node.read().await.name(), excluded_names)
+            .promote_and_demote_elders(&our_name, excluded_names)
             .await
         {
             commands.extend(self.send_dkg_start(elder_candidates).await?);
@@ -234,12 +232,12 @@ impl Core {
 
     pub(crate) async fn make_online_proposal(
         &self,
-        peer: &Peer,
+        peer: Peer,
         previous_name: Option<XorName>,
         dst_key: Option<bls::PublicKey>,
     ) -> Result<Vec<Command>> {
         self.propose(Proposal::Online {
-            node_state: NodeState::joined(peer, previous_name).into_msg(),
+            node_state: NodeState::joined(peer, previous_name),
             dst_key,
         })
         .await
