@@ -24,13 +24,12 @@ use crate::routing::{
     messages::{NodeMsgAuthorityUtils, WireMsgUtils},
     network_knowledge::NetworkKnowledge,
     node::Node,
-    Peer,
+    Peer, UnnamedPeer, MIN_ADULT_AGE,
 };
 use backoff::{backoff::Backoff, ExponentialBackoff};
 use bls::PublicKey as BlsPublicKey;
 use futures::future;
 use resource_proof::ResourceProof;
-use std::net::SocketAddr;
 use tokio::{sync::mpsc, time::Duration};
 use tracing::Instrument;
 use xor_name::Prefix;
@@ -44,7 +43,7 @@ pub(crate) async fn join_network(
     node: Node,
     comm: &Comm,
     incoming_conns: &mut mpsc::Receiver<ConnectionEvent>,
-    bootstrap_addr: SocketAddr,
+    bootstrap_peer: UnnamedPeer,
     genesis_key: BlsPublicKey,
 ) -> Result<(Node, NetworkKnowledge)> {
     let (send_tx, send_rx) = mpsc::channel(1);
@@ -56,7 +55,7 @@ pub(crate) async fn join_network(
 
     let state = Join::new(node, send_tx, incoming_conns, prefix_map);
 
-    future::join(state.run(bootstrap_addr), send_messages(send_rx, comm))
+    future::join(state.run(bootstrap_peer), send_messages(send_rx, comm))
         .instrument(span)
         .await
         .0
@@ -101,26 +100,34 @@ impl<'a> Join<'a> {
     // - `ResourceChallenge`: carry out resource proof calculation.
     // - `Approval`: returns the initial `Section` value to use by this node,
     //    completing the bootstrap.
-    async fn run(self, bootstrap_addr: SocketAddr) -> Result<(Node, NetworkKnowledge)> {
+    async fn run(self, bootstrap_peer: UnnamedPeer) -> Result<(Node, NetworkKnowledge)> {
         // Use our XorName as we do not know their name or section key yet.
-        let dst_xorname = self.node.name();
-        let recipients = vec![Peer::new(dst_xorname, bootstrap_addr)];
+        let bootstrap_peer = bootstrap_peer.named(self.node.name());
         let genesis_key = self.prefix_map.genesis_key();
 
-        self.join(genesis_key, recipients).await
+        let (target_section_key, recipients) =
+            if let Ok(sap) = self.prefix_map.section_by_name(&bootstrap_peer.name()) {
+                sap.merge_connections([&bootstrap_peer]).await;
+                (sap.section_key(), sap.elders_vec())
+            } else {
+                (genesis_key, vec![bootstrap_peer])
+            };
+
+        self.join(genesis_key, target_section_key, recipients).await
     }
 
     #[tracing::instrument(skip(self))]
     async fn join(
         mut self,
         network_genesis_key: BlsPublicKey,
+        target_section_key: BlsPublicKey,
         recipients: Vec<Peer>,
     ) -> Result<(Node, NetworkKnowledge)> {
         // We first use genesis key as the target section key, we'll be getting
         // a response with the latest section key for us to retry with.
         // Once we are approved to join, we will make sure the SAP we receive can
         // be validated with the received proof chain and the 'network_genesis_key'.
-        let mut section_key = network_genesis_key;
+        let mut section_key = target_section_key;
 
         // We send a first join request to obtain the resource challenge, which
         // we will then use to generate the challenge proof and send the
@@ -155,27 +162,36 @@ impl<'a> Join<'a> {
                     section_auth,
                     genesis_key,
                     section_chain,
-                    ..
+                    node_state,
                 } => {
-                    if genesis_key != network_genesis_key {
-                        debug!(
-                            "Ignoring JoinResponse::Approval with wrong network genesis key: {}, expected: {}",
-                            hex::encode(genesis_key.to_bytes()), hex::encode(network_genesis_key.to_bytes())
+                    trace!("{}", LogMarker::ReceivedJoinApproved);
+                    if node_state.name != self.node.name() {
+                        trace!("Ignore NodeApproval not for us: {:?}", node_state);
+                        continue;
+                    }
+
+                    if !node_state.verify(&section_chain) {
+                        error!(
+                            "Verification of node state in JoinResponse failed: {:?}",
+                            node_state
                         );
                         continue;
                     }
 
-                    trace!("{}", LogMarker::ReceivedJoinApproved);
+                    trace!(
+                        "This node has been approved to join the network at {:?}!",
+                        section_auth.prefix,
+                    );
 
-                    return Ok((
-                        self.node,
-                        NetworkKnowledge::new(
-                            genesis_key,
-                            section_chain,
-                            section_auth.into_authed_state(),
-                            Some(self.prefix_map),
-                        )?,
-                    ));
+                    // Building our network knowledge instance will validate SAP and section chain.
+                    let network_knowledge = NetworkKnowledge::new(
+                        genesis_key,
+                        section_chain,
+                        section_auth.into_authed_state(),
+                        Some(self.prefix_map),
+                    )?;
+
+                    return Ok((self.node, network_knowledge));
                 }
                 JoinResponse::Retry {
                     section_auth,
@@ -186,11 +202,13 @@ impl<'a> Join<'a> {
                     let section_auth = section_auth.into_state();
 
                     trace!(
-                        "Joining node {:?} - {:?}/{:?} received a Retry with SAP {:?}",
+                        "Joining node {:?} - {:?}/{:?} received a Retry from {} with SAP {:?}, expected_age: {}, our age: {}",
                         self.prefix,
                         self.node.name(),
-                        self.node.age(),
-                        section_auth
+                        self.node.age(),sender,
+                        section_auth,
+                        expected_age,
+                        self.node.age()
                     );
 
                     let prefix = section_auth.prefix();
@@ -210,15 +228,8 @@ impl<'a> Join<'a> {
                     };
 
                     // make sure we received a valid and trusted new SAP
-                    match self.prefix_map.update(signed_sap, &proof_chain) {
-                        Ok(true) => {}
-                        Ok(false) => {
-                            // it's not a new SAP, ignore it unless the expected age is different.
-                            if self.node.age() == expected_age {
-                                debug!("Ignoring JoinResponse::Retry with same SAP as we previously sent to: {:?}", section_auth);
-                                continue;
-                            }
-                        }
+                    let is_new_sap = match self.prefix_map.update(signed_sap, &proof_chain) {
+                        Ok(updated) => updated,
                         Err(err) => {
                             debug!(
                                 "Ignoring JoinResponse::Retry with an invalid SAP: {:?}",
@@ -226,12 +237,18 @@ impl<'a> Join<'a> {
                             );
                             continue;
                         }
-                    }
+                    };
 
-                    // make sure our joining age is the expected by the network
-                    if self.node.age() != expected_age {
-                        trace!("re-generate name due to mis-matched age, current {:?} vs. expected {:?}",
-                            self.node.age(), expected_age);
+                    // if it's not a new SAP, ignore response unless the expected age is different.
+                    if self.node.age() != expected_age
+                        && (self.node.age() > expected_age || self.node.age() == MIN_ADULT_AGE)
+                    {
+                        // adjust our joining age to the expected by the network
+                        trace!(
+                            "Re-generating name due to mis-matched age, current {} vs. expected {}",
+                            self.node.age(),
+                            expected_age
+                        );
                         let new_keypair = ed25519::gen_keypair(
                             &Prefix::default().range_inclusive(),
                             expected_age,
@@ -240,6 +257,9 @@ impl<'a> Join<'a> {
 
                         info!("Setting Node name to {} (age {})", new_name, expected_age);
                         self.node = Node::new(new_keypair, self.node.addr);
+                    } else if !is_new_sap {
+                        debug!("Ignoring JoinResponse::Retry with same SAP as we previously sent to: {:?}", section_auth);
+                        continue;
                     }
 
                     info!(
@@ -255,13 +275,24 @@ impl<'a> Join<'a> {
                         resource_proof_response: None,
                     };
 
-                    let new_recipients = section_auth.peers();
+                    section_auth
+                        .merge_connections(recipients.iter().chain([&sender]))
+                        .await;
+                    let new_recipients = section_auth.elders_vec();
                     self.send_join_requests(join_request, &new_recipients, section_key, true)
                         .await?;
                 }
                 JoinResponse::Redirect(section_auth) => {
-                    let section_auth = section_auth.into_state();
+                    trace!("Received a redirect/retry JoinResponse from {}. Sending request to the latest contacts", sender);
+                    if section_auth.elders.is_empty() {
+                        error!(
+                            "Invalid JoinResponse::Redirect, empty list of Elders: {:?}",
+                            section_auth
+                        );
+                        continue;
+                    }
 
+                    let section_auth = section_auth.into_state();
                     if !section_auth.prefix().matches(&self.node.name()) {
                         warn!(
                             "Ignoring newer JoinResponse::Redirect response not for us {:?}, SAP {:?} from {:?}",
@@ -273,11 +304,10 @@ impl<'a> Join<'a> {
                     }
 
                     let new_section_key = section_auth.section_key();
-
                     let new_recipients: Vec<_> = section_auth
-                        .peers()
-                        .into_iter()
+                        .elders()
                         .filter(|peer| used_recipient_saps.insert((peer.addr(), new_section_key)))
+                        .cloned()
                         .collect();
 
                     if new_recipients.is_empty() {
@@ -303,6 +333,9 @@ impl<'a> Join<'a> {
                         resource_proof_response: None,
                     };
 
+                    section_auth
+                        .merge_connections(recipients.iter().chain([&sender]))
+                        .await;
                     self.send_join_requests(join_request, &new_recipients, section_key, true)
                         .await?;
                 }
@@ -312,6 +345,7 @@ impl<'a> Join<'a> {
                     nonce,
                     nonce_signature,
                 } => {
+                    trace!("Received a ResourceChallenge from {}", sender);
                     let rp = ResourceProof::new(data_size, difficulty);
                     let data = rp.create_proof_data(&nonce);
                     let mut prover = rp.create_prover(data.clone());
@@ -351,7 +385,7 @@ impl<'a> Join<'a> {
             } else {
                 error!("Waiting before attempting to join again");
 
-                tokio::time::sleep(2 * self.backoff.max_interval).await;
+                tokio::time::sleep(self.backoff.max_interval).await;
                 self.backoff.reset();
             }
         }
@@ -379,15 +413,15 @@ impl<'a> Join<'a> {
     #[tracing::instrument(skip(self))]
     async fn receive_join_response(&mut self) -> Result<(JoinResponse, Peer)> {
         while let Some(event) = self.recv_rx.recv().await {
-            // we are interested only in `JoinResponse` type of messages
+            // We are interested only in `JoinResponse` type of messages
             let (join_response, sender) = match event {
-                ConnectionEvent::Received((sender_addr, bytes)) => match WireMsg::from(bytes) {
+                ConnectionEvent::Received((sender, bytes)) => match WireMsg::from(bytes) {
                     Ok(wire_msg) => match wire_msg.msg_kind() {
                         MsgKind::ServiceMsg(_) => continue,
                         MsgKind::NodeBlsShareAuthMsg(_) => {
                             trace!(
                                 "Bootstrap message discarded: sender: {:?} wire_msg: {:?}",
-                                sender_addr,
+                                sender,
                                 wire_msg
                             );
                             continue;
@@ -397,17 +431,14 @@ impl<'a> Join<'a> {
                                 msg: SystemMsg::JoinResponse(resp),
                                 msg_authority,
                                 ..
-                            }) => (
-                                *resp,
-                                Peer::new(msg_authority.src_location().name(), sender_addr),
-                            ),
+                            }) => (*resp, sender.named(msg_authority.src_location().name())),
                             Ok(
                                 MessageType::Service { msg_id, .. }
                                 | MessageType::System { msg_id, .. },
                             ) => {
                                 trace!(
                                     "Bootstrap message discarded: sender: {:?} msg_id: {:?}",
-                                    sender_addr,
+                                    sender,
                                     msg_id
                                 );
                                 continue;
@@ -425,59 +456,7 @@ impl<'a> Join<'a> {
                 },
             };
 
-            match join_response {
-                JoinResponse::ResourceChallenge { .. }
-                | JoinResponse::Rejected(JoinRejectionReason::NodeNotReachable(_))
-                | JoinResponse::Rejected(JoinRejectionReason::JoinsDisallowed) => {
-                    return Ok((join_response, sender));
-                }
-                JoinResponse::Retry {
-                    ref section_auth, ..
-                }
-                | JoinResponse::Redirect(ref section_auth) => {
-                    if section_auth.elders.is_empty() {
-                        error!(
-                            "Invalid JoinResponse::Retry/Redirect, empty list of Elders: {:?}",
-                            join_response
-                        );
-                        continue;
-                    }
-                    trace!("Received a redirect/retry JoinResponse from {}. Sending request to the latest contacts", sender);
-
-                    return Ok((join_response, sender));
-                }
-                JoinResponse::Approval {
-                    ref section_auth,
-                    ref node_state,
-                    ref section_chain,
-                    ..
-                } => {
-                    if node_state.name != self.node.name() {
-                        trace!("Ignore NodeApproval not for us");
-                        continue;
-                    }
-
-                    if !section_auth.verify(section_chain) {
-                        error!(
-                            "Verification of section authority failed: {:?}",
-                            join_response
-                        );
-                        continue;
-                    }
-
-                    if !node_state.verify(section_chain) {
-                        error!("Verification of node state failed: {:?}", join_response);
-                        continue;
-                    }
-
-                    trace!(
-                        "This node has been approved to join the network at {:?}!",
-                        section_auth.prefix,
-                    );
-
-                    return Ok((join_response, sender));
-                }
-            }
+            return Ok((join_response, sender));
         }
 
         error!("NodeMsg sender unexpectedly closed");
@@ -517,8 +496,10 @@ mod tests {
         error::Error as RoutingError,
         messages::WireMsgUtils,
         network_knowledge::{test_utils::*, NodeState},
-        ELDER_SIZE, MIN_ADULT_AGE,
+        UnnamedPeer, MIN_ADULT_AGE,
     };
+    use crate::{elder_count, init_test_logger};
+
     use crate::types::PublicKey;
     use assert_matches::assert_matches;
     use eyre::{eyre, Error, Result};
@@ -527,7 +508,7 @@ mod tests {
         pin_mut,
     };
     use secured_linked_list::SecuredLinkedList;
-    use std::collections::BTreeMap;
+    use std::{collections::BTreeMap, net::SocketAddr};
     use tokio::task;
     use xor_name::XorName;
 
@@ -537,7 +518,7 @@ mod tests {
         let (recv_tx, mut recv_rx) = mpsc::channel(1);
 
         let (section_auth, mut nodes, sk_set) =
-            gen_section_authority_provider(Prefix::default(), ELDER_SIZE);
+            gen_section_authority_provider(Prefix::default(), elder_count());
         let bootstrap_node = nodes.remove(0);
         let bootstrap_addr = bootstrap_node.addr;
         let sk = sk_set.secret_key();
@@ -559,7 +540,12 @@ mod tests {
         );
 
         // Create the bootstrap task, but don't run it yet.
-        let bootstrap = async move { state.run(bootstrap_addr).await.map_err(Error::from) };
+        let bootstrap = async move {
+            state
+                .run(UnnamedPeer::addressed(bootstrap_addr))
+                .await
+                .map_err(Error::from)
+        };
 
         // Create the task that executes the body of the test, but don't run it either.
         let others = async {
@@ -589,7 +575,7 @@ mod tests {
             send_response(
                 &recv_tx,
                 SystemMsg::JoinResponse(Box::new(JoinResponse::Retry {
-                    section_auth: section_auth.clone().into_msg(),
+                    section_auth: section_auth.to_msg(),
                     section_signed: signed_sap.sig,
                     proof_chain: section_chain,
                     expected_age: MIN_ADULT_AGE,
@@ -607,14 +593,14 @@ mod tests {
                 (msg, dst_location));
 
             assert_eq!(dst_location.section_pk(), Some(section_key));
-            itertools::assert_equal(recipients, section_auth.peers());
+            itertools::assert_equal(recipients, section_auth.elders());
             assert_matches!(node_msg, SystemMsg::JoinRequest(request) => {
                 assert_eq!(request.section_key, section_key);
             });
 
             // Send JoinResponse::Approval
             let section_auth = section_signed(sk, section_auth.clone())?;
-            let node_state = section_signed(sk, NodeState::joined(&peer, None))?;
+            let node_state = section_signed(sk, NodeState::joined(peer, None))?;
             let proof_chain = SecuredLinkedList::new(section_key);
             send_response(
                 &recv_tx,
@@ -646,7 +632,8 @@ mod tests {
         let (send_tx, mut send_rx) = mpsc::channel(1);
         let (recv_tx, mut recv_rx) = mpsc::channel(1);
 
-        let (_, mut nodes, sk_set) = gen_section_authority_provider(Prefix::default(), ELDER_SIZE);
+        let (_, mut nodes, sk_set) =
+            gen_section_authority_provider(Prefix::default(), elder_count());
         let bootstrap_node = nodes.remove(0);
         let genesis_key = sk_set.secret_key().public_key();
 
@@ -661,7 +648,7 @@ mod tests {
             NetworkPrefixMap::new(genesis_key),
         );
 
-        let bootstrap_task = state.run(bootstrap_node.addr);
+        let bootstrap_task = state.run(UnnamedPeer::addressed(bootstrap_node.addr));
         let test_task = async move {
             // Receive JoinRequest
             let (wire_msg, recipients) = send_rx
@@ -681,12 +668,12 @@ mod tests {
                     assert_matches!(msg, SystemMsg::JoinRequest{..}));
 
             // Send JoinResponse::Redirect
-            let new_bootstrap_addrs: BTreeMap<_, _> = (0..ELDER_SIZE)
+            let new_bootstrap_addrs: BTreeMap<_, _> = (0..elder_count())
                 .map(|_| (XorName::random(), gen_addr()))
                 .collect();
 
             let (new_section_auth, _, new_sk_set) =
-                gen_section_authority_provider(Prefix::default(), ELDER_SIZE);
+                gen_section_authority_provider(Prefix::default(), elder_count());
             let new_pk_set = new_sk_set.public_keys();
 
             send_response(
@@ -742,13 +729,14 @@ mod tests {
 
     #[tokio::test(flavor = "multi_thread")]
     async fn join_invalid_redirect_response() -> Result<()> {
-        crate::init_test_logger();
+        init_test_logger();
         let _span = tracing::info_span!("join_invalid_redirect_response").entered();
 
         let (send_tx, mut send_rx) = mpsc::channel(1);
         let (recv_tx, mut recv_rx) = mpsc::channel(1);
 
-        let (_, mut nodes, sk_set) = gen_section_authority_provider(Prefix::default(), ELDER_SIZE);
+        let (_, mut nodes, sk_set) =
+            gen_section_authority_provider(Prefix::default(), elder_count());
         let bootstrap_node = nodes.remove(0);
 
         let node = Node::new(
@@ -763,7 +751,7 @@ mod tests {
             NetworkPrefixMap::new(section_key),
         );
 
-        let bootstrap_task = state.run(bootstrap_node.addr);
+        let bootstrap_task = state.run(UnnamedPeer::addressed(bootstrap_node.addr));
         let test_task = async {
             let (wire_msg, _) = send_rx
                 .recv()
@@ -774,7 +762,7 @@ mod tests {
             assert_matches!(msg, SystemMsg::JoinRequest{..}));
 
             let (new_section_auth, _, new_sk_set) =
-                gen_section_authority_provider(Prefix::default(), ELDER_SIZE);
+                gen_section_authority_provider(Prefix::default(), elder_count());
             let new_pk_set = new_sk_set.public_keys();
 
             send_response(
@@ -791,7 +779,7 @@ mod tests {
             )?;
             task::yield_now().await;
 
-            let addrs = (0..ELDER_SIZE)
+            let addrs = (0..elder_count())
                 .map(|_| (XorName::random(), gen_addr()))
                 .collect();
 
@@ -835,7 +823,7 @@ mod tests {
         let (recv_tx, mut recv_rx) = mpsc::channel(1);
 
         let (section_auth, mut nodes, sk_set) =
-            gen_section_authority_provider(Prefix::default(), ELDER_SIZE);
+            gen_section_authority_provider(Prefix::default(), elder_count());
         let bootstrap_node = nodes.remove(0);
 
         let node = Node::new(
@@ -851,7 +839,7 @@ mod tests {
             NetworkPrefixMap::new(section_key),
         );
 
-        let bootstrap_task = state.run(bootstrap_node.addr);
+        let bootstrap_task = state.run(UnnamedPeer::addressed(bootstrap_node.addr));
         let test_task = async {
             let (wire_msg, _) = send_rx
                 .recv()
@@ -885,7 +873,7 @@ mod tests {
 
     #[tokio::test(flavor = "multi_thread")]
     async fn join_invalid_retry_prefix_response() -> Result<()> {
-        crate::init_test_logger();
+        init_test_logger();
         let _span = tracing::info_span!("join_invalid_retry_prefix_response").entered();
 
         let (send_tx, mut send_rx) = mpsc::channel(1);
@@ -912,7 +900,7 @@ mod tests {
             }
         };
 
-        let (section_auth, _, sk_set) = gen_section_authority_provider(good_prefix, ELDER_SIZE);
+        let (section_auth, _, sk_set) = gen_section_authority_provider(good_prefix, elder_count());
         let section_key = sk_set.public_keys().public_key();
 
         let state = Join::new(
@@ -922,10 +910,10 @@ mod tests {
             NetworkPrefixMap::new(section_key),
         );
 
-        let elders = (0..ELDER_SIZE)
+        let elders = (0..elder_count())
             .map(|_| Peer::new(good_prefix.substituted_in(rand::random()), gen_addr()))
             .collect();
-        let join_task = state.join(section_key, elders);
+        let join_task = state.join(section_key, section_key, elders);
 
         let test_task = async {
             let (wire_msg, _) = send_rx
@@ -944,9 +932,9 @@ mod tests {
             send_response(
                 &recv_tx,
                 SystemMsg::JoinResponse(Box::new(JoinResponse::Retry {
-                    section_auth: gen_section_authority_provider(bad_prefix, ELDER_SIZE)
+                    section_auth: gen_section_authority_provider(bad_prefix, elder_count())
                         .0
-                        .into_msg(),
+                        .to_msg(),
                     section_signed: signed_sap.sig.clone(),
                     proof_chain: section_chain.clone(),
                     expected_age: MIN_ADULT_AGE,
@@ -960,7 +948,7 @@ mod tests {
             send_response(
                 &recv_tx,
                 SystemMsg::JoinResponse(Box::new(JoinResponse::Retry {
-                    section_auth: section_auth.into_msg(),
+                    section_auth: section_auth.to_msg(),
                     section_signed: signed_sap.sig,
                     proof_chain: section_chain,
                     expected_age: MIN_ADULT_AGE,
@@ -1011,7 +999,7 @@ mod tests {
         debug!("wire msg built");
 
         recv_tx.try_send(ConnectionEvent::Received((
-            bootstrap_node.addr,
+            UnnamedPeer::addressed(bootstrap_node.addr),
             wire_msg.serialize()?,
         )))?;
 
