@@ -8,8 +8,9 @@
 
 use std::{cmp, collections::BTreeSet};
 
-use crate::messaging::system::{KeyedSig, MembershipState, Proposal, SectionAuth};
+use crate::messaging::system::{KeyedSig, MembershipState, SectionAuth};
 use crate::routing::{
+    core::Proposal,
     dkg::SectionAuthUtils,
     error::Result,
     log_markers::LogMarker,
@@ -23,7 +24,7 @@ use super::Core;
 // Agreement
 impl Core {
     #[instrument(skip(self), level = "trace")]
-    pub(crate) async fn handle_non_elder_agreement(
+    pub(crate) async fn handle_general_agreements(
         &self,
         proposal: Proposal,
         sig: KeyedSig,
@@ -31,16 +32,11 @@ impl Core {
         debug!("handle agreement on {:?}", proposal);
         match proposal {
             Proposal::Online { node_state, .. } => {
-                self.handle_online_agreement(node_state.into_state(), sig)
-                    .await
-            }
-            Proposal::Offline(node_state) => {
-                self.handle_offline_agreement(node_state.into_state(), sig)
-                    .await
+                self.handle_online_agreement(node_state, sig).await
             }
+            Proposal::Offline(node_state) => self.handle_offline_agreement(node_state, sig).await,
             Proposal::SectionInfo(section_auth) => {
-                self.handle_section_info_agreement(section_auth.into_state(), sig)
-                    .await
+                self.handle_section_info_agreement(section_auth, sig).await
             }
             Proposal::OurElders(_) => {
                 error!("Elders agreement should be handled in a separate blocking fashion");
@@ -84,7 +80,7 @@ impl Core {
                 // having to send this `NodeApproval`.
                 commands.extend(self.send_node_approval(old_info.clone()).await);
 
-                let peer = new_info.to_peer();
+                let peer = new_info.peer().clone();
                 commands.extend(self.relocate_rejoining_peer(peer, new_age).await?);
 
                 return Ok(commands);
@@ -103,6 +99,7 @@ impl Core {
 
         info!("handle Online: {} at {}", new_info.name(), new_info.addr());
 
+        // still used for testing
         self.send_event(Event::MemberJoined {
             name: new_info.name(),
             previous_name: new_info.previous_name(),
@@ -110,6 +107,15 @@ impl Core {
         })
         .await;
 
+        self.log_network_stats().await;
+
+        if new_info.previous_name().is_some() {
+            // Switch joins_allowed off a new adult joining.
+            *self.joins_allowed.write().await = false;
+        } else if !self.network_knowledge.prefix().await.is_empty() {
+            *self.joins_allowed.write().await = true;
+        }
+
         commands.extend(
             self.relocate_peers(&new_info.name(), &new_info.sig.signature)
                 .await?,
@@ -128,13 +134,13 @@ impl Core {
         Ok(commands)
     }
 
+    #[instrument(skip(self))]
     async fn handle_offline_agreement(
         &self,
         node_state: NodeState,
         sig: KeyedSig,
     ) -> Result<Vec<Command>> {
         let mut commands = vec![];
-        let age = node_state.age();
         let signature = sig.signature.clone();
 
         if !self
@@ -168,11 +174,16 @@ impl Core {
 
         commands.extend(result);
 
-        self.send_event(Event::MemberLeft {
-            name: node_state.name(),
-            age,
-        })
-        .await;
+        self.retain_members_only(
+            self.network_knowledge
+                .adults()
+                .await
+                .iter()
+                .map(|peer| peer.name())
+                .collect(),
+        )
+        .await?;
+        *self.joins_allowed.write().await = true;
 
         Ok(commands)
     }
@@ -189,67 +200,35 @@ impl Core {
                 .is_extension_of(&self.network_knowledge.prefix().await);
 
         if equal_or_extension {
+            // Our section or sub-section
             debug!(
                 "Updating section info for our prefix: {:?}",
                 section_auth.prefix()
             );
-            // Our section or sub-section
+
             let signed_section_auth = SectionAuth::new(section_auth, sig.clone());
-            let infos = self
+            let saps_candidates = self
                 .network_knowledge
                 .promote_and_demote_elders(&self.node.read().await.name(), &BTreeSet::new())
                 .await;
-            if !infos.contains(&signed_section_auth.elder_candidates()) {
+
+            if !saps_candidates.contains(&signed_section_auth.elder_candidates()) {
                 // SectionInfo out of date, ignore.
                 return Ok(vec![]);
             }
 
-            // Send a `AE Update` message to all the to-be-promoted members so they have the full
-            // section and network data.
-            let mut ae_update_recipients = vec![];
-
-            let mut peers = vec![];
-
-            for elder_candidate in infos.clone() {
-                peers.extend(elder_candidate.elders().cloned())
-            }
-
-            for peer in peers {
-                if !self.network_knowledge.is_elder(&peer.name()).await {
-                    ae_update_recipients.push(peer);
-                }
-            }
-
-            let mut commands = vec![];
-            if !ae_update_recipients.is_empty() {
-                let node_msg = self.generate_ae_update(sig.public_key, true).await?;
-                let cmd = self
-                    .send_direct_message_to_nodes(
-                        ae_update_recipients,
-                        node_msg,
-                        self.network_knowledge.prefix().await.name(),
-                        sig.public_key,
-                    )
-                    .await?;
-
-                commands.push(cmd);
-            }
-
-            // Send the `OurElder` proposal to all of the to-be-elders so it's aggregated by them.
-            let our_elders_recipients: Vec<_> = infos
+            // Send the `OurElder` proposal to all of the to-be-Elders so it's aggregated by them.
+            let proposal_recipients = saps_candidates
                 .iter()
                 .flat_map(|info| info.elders())
                 .cloned()
                 .collect();
-            commands.extend(
-                self.send_proposal(
-                    our_elders_recipients,
-                    Proposal::OurElders(signed_section_auth.into_authed_msg()),
-                )
-                .await?,
-            );
 
-            Ok(commands)
+            self.send_proposal(
+                proposal_recipients,
+                Proposal::OurElders(signed_section_auth),
+            )
+            .await
         } else {
             // Other section. We shouln't be receiving or updating a SAP for
             // a remote section here, that is done with a AE msg response.
