@@ -7,10 +7,10 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use super::super::Core;
-use crate::messaging::system::{
-    DkgFailureSig, DkgFailureSigSet, DkgSessionId, Proposal, SystemMsg,
-};
+use crate::messaging::system::{DkgFailureSig, DkgFailureSigSet, DkgSessionId, SystemMsg};
+use crate::messaging::DstLocation;
 use crate::routing::{
+    core::Proposal,
     dkg::DkgFailureSigSetUtils,
     error::{Error, Result},
     log_markers::LogMarker,
@@ -18,6 +18,7 @@ use crate::routing::{
     routing_api::command::Command,
     Peer,
 };
+use bls::PublicKey as BlsPublicKey;
 use bls_dkg::key_gen::message::Message as DkgMessage;
 use std::{
     collections::{BTreeMap, BTreeSet},
@@ -32,9 +33,17 @@ impl Core {
         prefix: Prefix,
         elders: BTreeMap<XorName, SocketAddr>,
     ) -> Result<Vec<Command>> {
+        let section_peers = self.network_knowledge().members();
         let elder_candidates = ElderCandidates::new(
             prefix,
-            elders.into_iter().map(|(name, addr)| Peer::new(name, addr)),
+            elders.into_iter().map(|(name, addr)| {
+                // Reuse known peers from network_knowledge, in order to preserve connections
+                if let Some(node) = section_peers.get(&name).filter(|node| node.addr() == addr) {
+                    node.peer().clone()
+                } else {
+                    Peer::new(name, addr)
+                }
+            }),
         );
         trace!("Received DkgStart for {:?}", elder_candidates);
         self.dkg_voter
@@ -62,6 +71,7 @@ impl Core {
 
         self.dkg_voter
             .process_message(
+                sender,
                 &self.node.read().await.clone(),
                 &session_id,
                 message,
@@ -70,6 +80,63 @@ impl Core {
             .await
     }
 
+    pub(crate) fn handle_dkg_not_ready(
+        &self,
+        sender: Peer,
+        message: DkgMessage,
+        session_id: DkgSessionId,
+        section_pk: BlsPublicKey,
+    ) -> Vec<Command> {
+        let message_history = self.dkg_voter.get_cached_messages(&session_id);
+        vec![Command::PrepareNodeMsgToSend {
+            msg: SystemMsg::DkgRetry {
+                message_history,
+                message,
+                session_id,
+            },
+            dst: DstLocation::Node {
+                name: sender.name(),
+                section_pk,
+            },
+        }]
+    }
+
+    pub(crate) async fn handle_dkg_retry(
+        &self,
+        session_id: DkgSessionId,
+        message_history: Vec<DkgMessage>,
+        message: DkgMessage,
+        sender: XorName,
+    ) -> Result<Vec<Command>> {
+        let sender_pk = self
+            .network_knowledge()
+            .section_by_name(&sender)?
+            .section_key();
+        let mut commands = vec![];
+        commands.extend(
+            self.dkg_voter
+                .handle_dkg_history(
+                    &self.node.read().await.clone(),
+                    session_id,
+                    message_history,
+                    sender_pk,
+                )
+                .await?,
+        );
+        commands.extend(
+            self.dkg_voter
+                .process_message(
+                    sender,
+                    &self.node.read().await.clone(),
+                    &session_id,
+                    message,
+                    sender_pk,
+                )
+                .await?,
+        );
+        Ok(commands)
+    }
+
     pub(crate) fn handle_dkg_failure_observation(
         &self,
         session_id: DkgSessionId,
@@ -138,13 +205,14 @@ impl Core {
 
     pub(crate) async fn handle_dkg_outcome(
         &self,
-        section_auth: SectionAuthorityProvider,
+        sap: SectionAuthorityProvider,
         key_share: SectionKeyShare,
     ) -> Result<Vec<Command>> {
+        let key_share_pk = key_share.public_key_set.public_key();
         trace!(
             "{} public_key={:?}",
             LogMarker::HandlingDkgSuccessfulOutcome,
-            key_share.public_key_set.public_key()
+            key_share_pk
         );
 
         // Add our new keyshare to our cache, we will then use
@@ -152,17 +220,24 @@ impl Core {
         self.section_keys_provider.insert(key_share.clone()).await;
 
         let snapshot = self.state_snapshot().await;
-        //
+
+        // If we are lagging, we may have been already approved as new Elder, and
+        // an AE update provided us with this same SAP but already signed by previous Elders,
+        // if so we can skip the SectionInfo agreement proposal phase.
         if self
             .network_knowledge
-            .skip_section_info_agreement(key_share.public_key_set.public_key())
+            .set_current_sap(key_share_pk, &sap.prefix())
             .await
         {
             self.update_self_for_new_node_state_and_fire_events(snapshot)
                 .await
         } else {
-            let proposal = Proposal::SectionInfo(section_auth.into_msg());
-            let recipients: Vec<_> = self.network_knowledge.authority_provider().await.peers();
+            let proposal = Proposal::SectionInfo(sap);
+            let recipients: Vec<_> = self
+                .network_knowledge
+                .authority_provider()
+                .await
+                .elders_vec();
             self.send_proposal_with(recipients, proposal, &key_share)
                 .await
         }
