@@ -80,7 +80,7 @@ impl Core {
             let retry_sap = self.matching_section(&peer.name()).await?;
 
             let node_msg =
-                SystemMsg::JoinResponse(Box::new(JoinResponse::Redirect(retry_sap.into_msg())));
+                SystemMsg::JoinResponse(Box::new(JoinResponse::Redirect(retry_sap.to_msg())));
 
             trace!("Sending {:?} to {}", node_msg, peer);
             trace!("{}", LogMarker::SendJoinRedirected);
@@ -111,19 +111,18 @@ impl Core {
         // During the first section, node shall use ranged age to avoid too many nodes got
         // relocated at the same time. After the first section got split, nodes shall only
         // start with age of MIN_ADULT_AGE
-        let expected_age: u8 = if our_prefix.is_empty() {
+        let (is_age_invalid, expected_age): (bool, u8) = if our_prefix.is_empty() {
             let section_members = self.network_knowledge.active_members().await.len();
-            // Calculate a deterministic value based on peer's address
-            // within the range [FIRST_SECTION_MIN_AGE, FIRST_SECTION_MAX_AGE].
-            // let value: u8 = bincode::serialize(&peer.addr())?.iter().sum();
-            // let range = FIRST_SECTION_MAX_AGE - FIRST_SECTION_MIN_AGE;
-
-            FIRST_SECTION_MAX_AGE - section_members as u8 * 2
+            // Calculate a value within the range [FIRST_SECTION_MIN_AGE, FIRST_SECTION_MAX_AGE].
+            let expected_age = FIRST_SECTION_MAX_AGE - section_members as u8 * 2;
+            let is_age_invalid = peer.age() == MIN_ADULT_AGE || peer.age() > expected_age;
+            (is_age_invalid, expected_age)
         } else {
-            MIN_ADULT_AGE
+            let is_age_invalid = peer.age() != MIN_ADULT_AGE;
+            (is_age_invalid, MIN_ADULT_AGE)
         };
 
-        if !section_key_matches || peer.age() != expected_age {
+        if !section_key_matches || is_age_invalid {
             if !section_key_matches {
                 trace!("{}", LogMarker::SendJoinRetryNotCorrectKey);
                 trace!(
@@ -135,23 +134,21 @@ impl Core {
             } else {
                 trace!("{}", LogMarker::SendJoinRetryAgeIssue);
                 trace!(
-                    "JoinReequest from {} doesn't have the expected age {:?}, expected age {}",
+                    "JoinRequest from {} (with age {}) doesn't have the expected: {}",
                     peer,
                     peer.age(),
                     expected_age,
                 );
             }
 
-            // TODO: once we keep a DAG for chains, return only a delta
-            // from provided section key rather than the chain from genesis key.
-            let proof_chain = self.network_knowledge.chain().await;
+            let proof_chain = self.network_knowledge.section_chain().await;
             let signed_sap = self
                 .network_knowledge
                 .section_signed_authority_provider()
                 .await;
 
             let node_msg = SystemMsg::JoinResponse(Box::new(JoinResponse::Retry {
-                section_auth: signed_sap.value.into_msg(),
+                section_auth: signed_sap.value.to_msg(),
                 section_signed: signed_sap.sig,
                 proof_chain,
                 expected_age,
@@ -226,7 +223,7 @@ impl Core {
                 ))
             } else {
                 SystemMsg::JoinAsRelocatedResponse(Box::new(JoinAsRelocatedResponse::Retry(
-                    self.network_knowledge.authority_provider().await.into_msg(),
+                    self.network_knowledge.authority_provider().await.to_msg(),
                 )))
             };
             trace!("{}", LogMarker::SendJoinAsRelocatedResponse);
@@ -253,7 +250,7 @@ impl Core {
 
             let node_msg =
                 SystemMsg::JoinAsRelocatedResponse(Box::new(JoinAsRelocatedResponse::Retry(
-                    self.network_knowledge.authority_provider().await.into_msg(),
+                    self.network_knowledge.authority_provider().await.to_msg(),
                 )));
 
             trace!("{} b", LogMarker::SendJoinAsRelocatedResponse);
