@@ -21,8 +21,8 @@ use crate::messaging::{
     data::{ServiceMsg, StorageLevel},
     signature_aggregator::Error as AggregatorError,
     system::{NodeCmd, NodeQuery, SystemMsg},
-    DstLocation, EndUser, MessageId, MessageType, MsgKind, NodeMsgAuthority, SectionAuth,
-    ServiceAuth, SrcLocation, WireMsg,
+    DstLocation, MessageId, MessageType, MsgKind, NodeMsgAuthority, SectionAuth, ServiceAuth,
+    WireMsg,
 };
 use crate::routing::{
     log_markers::LogMarker,
@@ -30,13 +30,13 @@ use crate::routing::{
     network_knowledge::SectionPeers,
     relocation::RelocateState,
     routing_api::command::Command,
-    Error, Event, MessageReceived, Peer, Result,
+    Error, Event, MessageReceived, Peer, Result, UnnamedPeer, MIN_LEVEL_WHEN_FULL,
 };
 use crate::types::{Chunk, Keypair, PublicKey};
 use bls::PublicKey as BlsPublicKey;
 use bytes::Bytes;
 use rand::rngs::OsRng;
-use std::{collections::BTreeSet, net::SocketAddr};
+use std::collections::BTreeSet;
 use xor_name::XorName;
 
 // Message handling
@@ -44,14 +44,15 @@ impl Core {
     #[instrument(skip(self, original_bytes))]
     pub(crate) async fn handle_message(
         &self,
-        sender_addr: SocketAddr,
+        sender: UnnamedPeer,
         wire_msg: WireMsg,
         original_bytes: Option<Bytes>,
     ) -> Result<Vec<Command>> {
         let mut cmds = vec![];
         trace!("handling msg");
+
         // Apply backpressure if needed.
-        if let Some(load_report) = self.comm.check_strain(sender_addr).await {
+        if let Some(load_report) = self.comm.check_strain(sender.addr()).await {
             let msg_src = wire_msg.msg_kind().src();
             cmds.push(Command::PrepareNodeMsgToSend {
                 msg: SystemMsg::BackPressure(load_report),
@@ -85,7 +86,7 @@ impl Core {
                 // based on our current knowledge of the network and sections chains.
                 let mut known_keys: Vec<BlsPublicKey> = self
                     .network_knowledge
-                    .chain()
+                    .section_chain()
                     .await
                     .keys()
                     .copied()
@@ -96,7 +97,7 @@ impl Core {
                 if !msg_authority.verify_src_section_key_is_known(&known_keys) {
                     warn!(
                         "Untrusted message ({:?}) dropped from {:?}: {:?} ",
-                        msg_id, sender_addr, msg
+                        msg_id, sender, msg
                     );
                     return Ok(cmds);
                 }
@@ -104,14 +105,11 @@ impl Core {
                 trace!(
                     "Trusted msg authority in message ({:?}) from {:?}: {:?}",
                     msg_id,
-                    sender_addr,
+                    sender,
                     msg
                 );
 
-                // TODO: msg_authority.peer() exists, but will error if authority is
-                // section/sectionshare. It's not clear if this would actually
-                // represent a valid peer.
-                let sender = Peer::new(msg_authority.name(), sender_addr);
+                let sender = sender.named(msg_authority.name());
 
                 // Let's check for entropy before we proceed further
                 // Adult nodes don't need to carry out entropy checking,
@@ -185,28 +183,21 @@ impl Core {
                     None => {
                         error!(
                             "Service msg has been dropped since {:?} is not a valid msg to send from a client {}.",
-                            msg, sender_addr
+                            msg, sender.addr()
                         );
                         return Ok(vec![]);
                     }
                 };
-                let user = match self.comm.get_peer_connection_id(&sender_addr).await {
-                    Some(name) => EndUser(name),
-                    None => {
-                        error!(
-                            "Service msg has been dropped since client connection id for {} was not found: {:?}",
-                            sender_addr, msg
-                        );
-                        return Ok(cmds);
-                    }
-                };
 
-                let sender = Peer::new(user.0, sender_addr);
-                let src_location = SrcLocation::EndUser(user);
+                let src_location = wire_msg.msg_kind().src();
+                let sender = sender.named(src_location.name());
 
                 if self.is_not_elder().await {
                     trace!("Redirecting from adult to section elders");
-                    cmds.push(self.ae_redirect(sender, &src_location, &wire_msg).await?);
+                    cmds.push(
+                        self.ae_redirect_to_our_elders(sender, &src_location, &wire_msg)
+                            .await?,
+                    );
                     return Ok(cmds);
                 }
 
@@ -237,7 +228,7 @@ impl Core {
                 }
 
                 cmds.extend(
-                    self.handle_service_message(msg_id, auth, msg, dst_location, user)
+                    self.handle_service_message(msg_id, auth, msg, dst_location, sender)
                         .await?,
                 );
 
@@ -392,12 +383,14 @@ impl Core {
             SystemMsg::AntiEntropyRedirect {
                 section_auth,
                 section_signed,
+                section_chain,
                 bounced_msg,
             } => {
                 trace!("Handling msg: AE-Redirect from {}: {:?}", sender, msg_id);
                 self.handle_anti_entropy_redirect_msg(
                     section_auth.into_state(),
                     section_signed,
+                    section_chain,
                     bounced_msg,
                     sender,
                 )
@@ -449,7 +442,7 @@ impl Core {
                 }
 
                 trace!("Handling msg: Propose from {}: {:?}", sender, msg_id);
-                self.handle_proposal(msg_id, proposal, sig_share, sender)
+                self.handle_proposal(msg_id, proposal.into_state(), sig_share, sender)
                     .await
             }
             SystemMsg::DkgStart {
@@ -485,56 +478,86 @@ impl Core {
                 trace!("Handling msg: Dkg-FailureObservation from {}", sender);
                 self.handle_dkg_failure_observation(session_id, &failed_participants, sig)
             }
+            SystemMsg::DkgNotReady {
+                message,
+                session_id,
+            } => {
+                let src = sender.name();
+                Ok(self.handle_dkg_not_ready(
+                    sender,
+                    message,
+                    session_id,
+                    self.network_knowledge.section_by_name(&src)?.section_key(),
+                ))
+            }
+            SystemMsg::DkgRetry {
+                message_history,
+                message,
+                session_id,
+            } => {
+                self.handle_dkg_retry(session_id, message_history, message, sender.name())
+                    .await
+            }
             // The following type of messages are all handled by upper sn_node layer.
-            // TODO: In the future the sn-node layer won't be receiving Events but just
-            // plugging in msg handlers.
-            SystemMsg::NodeCmd(node_cmd) => {
-                match node_cmd {
-                    NodeCmd::StoreChunk { chunk, .. } => {
-                        info!("Processing chunk write with MessageId: {:?}", msg_id);
-                        // There is no point in verifying a sig from a sender A or B here.
-                        let level_report = self.chunk_storage.store(&chunk).await?;
-                        return Ok(self.record_if_any(level_report).await);
-                    }
-                    NodeCmd::ReplicateChunk(chunk) => {
-                        info!(
-                            "Processing replicate chunk cmd with MessageId: {:?}",
-                            msg_id
-                        );
-
-                        return if self.is_elder().await {
-                            self.republish_chunk(chunk).await
-                        } else {
-                            // We are an adult here, so just store away!
-
-                            // TODO: should this be a cmd returned for threading?
-                            let level_report =
-                                self.chunk_storage.store_for_replication(chunk).await?;
-                            Ok(self.record_if_any(level_report).await)
-                        };
-                    }
-                    NodeCmd::RepublishChunk(chunk) => {
-                        info!(
-                            "Republishing chunk {:?} with MessageId {:?}",
-                            chunk.name(),
-                            msg_id
-                        );
-
-                        return self.republish_chunk(chunk).await;
-                    }
-                    _ => {
-                        self.send_event(Event::MessageReceived {
-                            msg_id,
-                            src: msg_authority.src_location(),
-                            dst: dst_location,
-                            msg: Box::new(MessageReceived::NodeCmd(node_cmd)),
-                        })
-                        .await;
-                    }
+            // TODO: In the future the sn-node layer won't be receiving Events
+            SystemMsg::NodeCmd(NodeCmd::RecordStorageLevel { node_id, level, .. }) => {
+                let changed = self.set_storage_level(&node_id, level).await;
+                if changed && level.value() == MIN_LEVEL_WHEN_FULL {
+                    // ..then we accept a new node in place of the full node
+                    *self.joins_allowed.write().await = true;
                 }
+                Ok(vec![])
+            }
+            SystemMsg::NodeCmd(NodeCmd::ReceiveExistingData { metadata }) => {
+                info!("Processing received DataExchange packet: {:?}", msg_id);
+
+                self.register_storage.update(metadata.reg_data)?;
+                self.update_chunks(metadata.chunk_data).await;
+                Ok(vec![])
+            }
+            SystemMsg::NodeCmd(NodeCmd::StoreChunk { chunk, .. }) => {
+                info!("Processing chunk write with MessageId: {:?}", msg_id);
+                // There is no point in verifying a sig from a sender A or B here.
+                let level_report = self.chunk_storage.store(&chunk).await?;
+                return Ok(self.record_if_any(level_report).await);
+            }
+            SystemMsg::NodeCmd(NodeCmd::ReplicateChunk(chunk)) => {
+                info!(
+                    "Processing replicate chunk cmd with MessageId: {:?}",
+                    msg_id
+                );
+
+                return if self.is_elder().await {
+                    self.republish_chunk(chunk).await
+                } else {
+                    // We are an adult here, so just store away!
+
+                    // TODO: should this be a cmd returned for threading?
+                    let level_report = self.chunk_storage.store_for_replication(chunk).await?;
+                    Ok(self.record_if_any(level_report).await)
+                };
+            }
+            SystemMsg::NodeCmd(NodeCmd::RepublishChunk(chunk)) => {
+                info!(
+                    "Republishing chunk {:?} with MessageId {:?}",
+                    chunk.name(),
+                    msg_id
+                );
+
+                return self.republish_chunk(chunk).await;
+            }
+            SystemMsg::NodeCmd(node_cmd) => {
+                self.send_event(Event::MessageReceived {
+                    msg_id,
+                    src: msg_authority.src_location(),
+                    dst: dst_location,
+                    msg: Box::new(MessageReceived::NodeCmd(node_cmd)),
+                })
+                .await;
 
                 Ok(vec![])
             }
+
             SystemMsg::NodeQuery(node_query) => {
                 match node_query {
                     // A request from EndUser - via elders - for locally stored chunk
