@@ -7,20 +7,20 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use crate::messaging::{
-    system::{DkgFailureSigSet, KeyedSig, Proposal, SystemMsg},
+    system::{DkgFailureSigSet, KeyedSig, SystemMsg},
     DstLocation, MessageId, NodeMsgAuthority, WireMsg,
 };
 use crate::routing::{
+    core::Proposal,
     network_knowledge::{NetworkKnowledge, SectionAuthorityProvider, SectionKeyShare},
     node::Node,
-    Peer, XorName,
+    Peer, UnnamedPeer, XorName,
 };
 use bls::PublicKey as BlsPublicKey;
 use bytes::Bytes;
 use custom_debug::Debug;
 use std::{
     fmt,
-    net::SocketAddr,
     sync::atomic::{AtomicU64, Ordering},
     time::Duration,
 };
@@ -32,9 +32,7 @@ pub(crate) enum Command {
     /// Handle `message` from `sender`.
     /// Holding the WireMsg that has been received from the network,
     HandleMessage {
-        // This is the only command that uses `SocketAddr` instead of `Peer`, because we haven't
-        // deserialized/verified the src at this stage.
-        sender_addr: SocketAddr,
+        sender: UnnamedPeer,
         wire_msg: WireMsg,
         #[debug(skip)]
         // original bytes to avoid reserializing for entropy checks
@@ -129,9 +127,19 @@ impl fmt::Display for Command {
             Command::HandleElderAgreement { .. } => write!(f, "HandleElderAgreement"),
             Command::HandleDkgOutcome { .. } => write!(f, "HandleDkgOutcome"),
             Command::HandleDkgFailure(_) => write!(f, "HandleDkgFailure"),
+            #[cfg(not(feature = "unstable-wiremsg-debuginfo"))]
             Command::SendMessage { wire_msg, .. } => {
                 write!(f, "SendMessage {:?}", wire_msg.msg_id())
             }
+            #[cfg(feature = "unstable-wiremsg-debuginfo")]
+            Command::SendMessage { wire_msg, .. } => {
+                write!(
+                    f,
+                    "SendMessage {:?} {:?}",
+                    wire_msg.msg_id(),
+                    wire_msg.payload_debug
+                )
+            }
             Command::ParseAndSendWireMsg(wire_msg) => {
                 write!(f, "ParseAndSendWireMsg {:?}", wire_msg.msg_id())
             }
