@@ -7,24 +7,15 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use super::{Command, Event};
-use crate::messaging::{
-    data::{ChunkDataExchange, StorageLevel},
-    system::{Proposal, SystemMsg},
-    DstLocation, EndUser, MsgKind, WireMsg,
-};
+use crate::messaging::{system::SystemMsg, DstLocation, EndUser, MsgKind, WireMsg};
 use crate::routing::{
-    core::{ChunkStore, RegisterStorage},
-    core::{Core, SendStatus},
+    core::{Core, Proposal, SendStatus},
     error::Result,
     log_markers::LogMarker,
-    messages::WireMsgUtils,
     network_knowledge::NetworkKnowledge,
     node::Node,
-    Error, Peer, Prefix, XorName,
+    Error, Peer,
 };
-use crate::types::PublicKey;
-use itertools::Itertools;
-use std::collections::BTreeSet;
 use std::{sync::Arc, time::Duration};
 use tokio::time::MissedTickBehavior;
 use tokio::{sync::watch, time};
@@ -57,27 +48,6 @@ impl Dispatcher {
         }
     }
 
-    pub(super) async fn get_register_storage(&self) -> RegisterStorage {
-        self.core.register_storage.clone()
-    }
-
-    pub(super) async fn get_chunk_storage(&self) -> ChunkStore {
-        self.core.chunk_storage.clone()
-    }
-
-    pub(super) async fn get_chunk_data_of(&self, prefix: &Prefix) -> ChunkDataExchange {
-        self.core.get_data_of(prefix).await
-    }
-
-    /// Returns whether the level changed or not.
-    pub(super) async fn set_storage_level(&self, node_id: &PublicKey, level: StorageLevel) -> bool {
-        self.core.set_storage_level(node_id, level).await
-    }
-
-    pub(super) async fn retain_members_only(&self, members: BTreeSet<XorName>) -> Result<()> {
-        self.core.retain_members_only(members).await
-    }
-
     /// Handles the given command and transitively any new commands that are
     /// produced during its handling. Trace logs will include the provided command id,
     /// and any sub-commands produced will have it as a common root cmd id.
@@ -230,22 +200,22 @@ impl Dispatcher {
                 self.core.prepare_node_msg(msg, dst).await
             }
             Command::HandleMessage {
-                sender_addr,
+                sender,
                 wire_msg,
                 original_bytes,
             } => {
                 self.core
-                    .handle_message(sender_addr, wire_msg, original_bytes)
+                    .handle_message(sender, wire_msg, original_bytes)
                     .await
             }
             Command::HandleTimeout(token) => self.core.handle_timeout(token).await,
             Command::HandleAgreement { proposal, sig } => {
-                self.core.handle_non_elder_agreement(proposal, sig).await
+                self.core.handle_general_agreements(proposal, sig).await
             }
             Command::HandleElderAgreement { proposal, sig } => match proposal {
                 Proposal::OurElders(section_auth) => {
                     self.core
-                        .handle_our_elders_agreement(section_auth.into_authed_state(), sig)
+                        .handle_our_elders_agreement(section_auth, sig)
                         .await
                 }
                 _ => {
@@ -297,37 +267,15 @@ impl Dispatcher {
                 dst_key,
             } => {
                 self.core
-                    .make_online_proposal(&peer, previous_name, dst_key)
+                    .make_online_proposal(peer, previous_name, dst_key)
                     .await
             }
             Command::ProposeOffline(name) => self.core.propose_offline(name).await,
-            Command::StartConnectivityTest(name) => {
-                let msg = {
-                    let core = &self.core;
-                    let node = core.node.read().await.clone();
-                    let section_pk = core.network_knowledge().section_key().await;
-                    WireMsg::single_src(
-                        &node,
-                        DstLocation::Section {
-                            name: node.name(),
-                            section_pk,
-                        },
-                        SystemMsg::StartConnectivityTest(name),
-                        section_pk,
-                    )?
-                };
-                let our_name = self.core.node.read().await.name();
-                let peers = self
-                    .core
-                    .network_knowledge()
-                    .active_members()
-                    .await
-                    .iter()
-                    .filter(|peer| peer.name() != name && peer.name() != our_name)
-                    .cloned()
-                    .collect_vec();
-                Ok(self.core.send_or_handle(msg, peers).await)
-            }
+            Command::StartConnectivityTest(name) => Ok(vec![
+                self.core
+                    .send_message_to_our_elders(SystemMsg::StartConnectivityTest(name))
+                    .await?,
+            ]),
             Command::TestConnectivity(name) => {
                 let mut commands = vec![];
                 if let Some(member_info) = self.core.network_knowledge().members().get(&name) {
@@ -449,7 +397,7 @@ impl Dispatcher {
     ) -> Result<()> {
         let previous_name = self.core.node.read().await.name();
         let new_keypair = new_node.keypair.clone();
-
+        let age = new_node.age();
         self.core.relocate(new_node, new_section).await?;
 
         self.core
@@ -459,6 +407,8 @@ impl Dispatcher {
             })
             .await;
 
+        info!("Relocated, our Age: {:?}", age);
+
         Ok(())
     }
 }
