@@ -23,9 +23,10 @@ use self::{
     event::{Elders, Event, NodeElderChange},
     event_stream::EventStream,
 };
-use crate::messaging::{data::StorageLevel, system::SystemMsg, DstLocation, WireMsg};
+use crate::dbs::UsedSpace;
+use crate::messaging::{system::SystemMsg, DstLocation, WireMsg};
 use crate::routing::{
-    core::{join_network, ChunkStore, Comm, ConnectionEvent, Core, RegisterStorage},
+    core::{join_network, Comm, ConnectionEvent, Core},
     ed25519,
     error::{Error, Result},
     log_markers::LogMarker,
@@ -34,7 +35,6 @@ use crate::routing::{
     node::Node,
     Peer, MIN_ADULT_AGE,
 };
-use crate::{dbs::UsedSpace, messaging::data::ChunkDataExchange};
 use ed25519_dalek::{PublicKey, Signature, Signer, KEYPAIR_LENGTH};
 
 use crate::types::PublicKey as TypesPublicKey;
@@ -151,7 +151,7 @@ impl Routing {
             let node_name = ed25519::name(&keypair.public);
             info!("{} Bootstrapping a new node.", node_name);
 
-            let (comm, bootstrap_addr) = Comm::bootstrap(
+            let (comm, bootstrap_peer) = Comm::bootstrap(
                 config.local_addr,
                 config
                     .bootstrap_nodes
@@ -168,7 +168,7 @@ impl Routing {
                 node_name,
                 std::process::id(),
                 comm.our_connection_info(),
-                bootstrap_addr,
+                bootstrap_peer.addr(),
                 genesis_key
             );
 
@@ -177,7 +177,7 @@ impl Routing {
                 joining_node,
                 &comm,
                 &mut connection_event_rx,
-                bootstrap_addr,
+                bootstrap_peer,
                 genesis_key,
             )
             .await?;
@@ -215,33 +215,6 @@ impl Routing {
         Ok((routing, event_stream))
     }
 
-    pub(crate) async fn get_register_storage(&self) -> RegisterStorage {
-        self.dispatcher.get_register_storage().await
-    }
-
-    pub(crate) async fn get_chunk_storage(&self) -> ChunkStore {
-        self.dispatcher.get_chunk_storage().await
-    }
-    pub(crate) async fn get_chunk_data_of(&self, prefix: &Prefix) -> ChunkDataExchange {
-        self.dispatcher.get_chunk_data_of(prefix).await
-    }
-
-    /// Returns whether the level changed or not.
-    pub(crate) async fn set_storage_level(
-        &self,
-        node_id: &TypesPublicKey,
-        level: StorageLevel,
-    ) -> bool {
-        self.dispatcher.set_storage_level(node_id, level).await
-    }
-    pub(crate) async fn retain_members_only(&self, members: BTreeSet<XorName>) -> Result<()> {
-        self.dispatcher.retain_members_only(members).await
-    }
-
-    pub(crate) async fn update_chunks(&self, chunks: ChunkDataExchange) {
-        self.dispatcher.core.update_chunks(chunks).await
-    }
-
     /// Sets the JoinsAllowed flag.
     pub async fn set_joins_allowed(&self, joins_allowed: bool) -> Result<()> {
         let command = Command::SetJoinsAllowed(joins_allowed);
@@ -341,7 +314,7 @@ impl Routing {
             .network_knowledge()
             .authority_provider()
             .await
-            .peers()
+            .elders_vec()
     }
 
     /// Returns the elders of our section sorted by their distance to `name` (closest first).
@@ -368,15 +341,6 @@ impl Routing {
             .collect()
     }
 
-    /// Returns our section's authority provider.
-    pub(crate) async fn our_section_auth(&self) -> SectionAuthorityProvider {
-        self.dispatcher
-            .core
-            .network_knowledge()
-            .authority_provider()
-            .await
-    }
-
     /// Returns the info about the section matching the name.
     pub async fn matching_section(&self, name: &XorName) -> Result<SectionAuthorityProvider> {
         self.dispatcher.core.matching_section(name).await
@@ -449,11 +413,11 @@ async fn handle_connection_events(
 ) {
     while let Some(event) = incoming_conns.recv().await {
         match event {
-            ConnectionEvent::Received((sender_addr, bytes)) => {
+            ConnectionEvent::Received((sender, bytes)) => {
                 trace!(
-                    "New message ({} bytes) received from: {}",
+                    "New message ({} bytes) received from: {:?}",
                     bytes.len(),
-                    sender_addr
+                    sender
                 );
 
                 // bytes.clone is cheap
@@ -467,21 +431,21 @@ async fn handle_connection_events(
 
                 let span = {
                     let core = &dispatcher.core;
-                    trace_span!("handle_message", name = %core.node.read().await.name(), %sender_addr, msg_id = ?wire_msg.msg_id())
+                    trace_span!("handle_message", name = %core.node.read().await.name(), ?sender, msg_id = ?wire_msg.msg_id())
                 };
                 let _span_guard = span.enter();
-                let len = bytes.len();
+
+                trace!(
+                    "{:?} from {:?} length {}",
+                    LogMarker::DispatchHandleMsgCmd,
+                    sender,
+                    bytes.len(),
+                );
                 let command = Command::HandleMessage {
-                    sender_addr,
+                    sender,
                     wire_msg,
                     original_bytes: Some(bytes),
                 };
-                trace!(
-                    "{:?} from {} length {}",
-                    LogMarker::DispatchHandleMsgCmd,
-                    sender_addr,
-                    len,
-                );
 
                 let _handle = dispatcher.clone().handle_commands(command, None).await;
             }
