@@ -12,14 +12,14 @@ use super::{Comm, Command, Core, Dispatcher};
 use crate::dbs::UsedSpace;
 use crate::messaging::{
     system::{
-        JoinAsRelocatedRequest, JoinRequest, JoinResponse, KeyedSig, MembershipState, Proposal,
+        JoinAsRelocatedRequest, JoinRequest, JoinResponse, KeyedSig, MembershipState,
         RelocateDetails, RelocatePayload, ResourceProofResponse, SectionAuth, SystemMsg,
     },
     AuthorityProof, DstLocation, MessageId, MessageType, MsgKind, NodeAuth,
     SectionAuth as MsgKindSectionAuth, WireMsg,
 };
 use crate::routing::{
-    core::{ConnectionEvent, ProposalUtils, RESOURCE_PROOF_DATA_SIZE, RESOURCE_PROOF_DIFFICULTY},
+    core::{ConnectionEvent, Proposal, RESOURCE_PROOF_DATA_SIZE, RESOURCE_PROOF_DIFFICULTY},
     create_test_used_space_and_root_storage,
     dkg::test_utils::{prove, section_signed},
     ed25519,
@@ -29,9 +29,11 @@ use crate::routing::{
     },
     node::Node,
     relocation::{self, RelocatePayloadUtils},
-    supermajority, Error, Event, Peer, Result as RoutingResult, ELDER_SIZE, FIRST_SECTION_MAX_AGE,
+    supermajority, Error, Event, Peer, Result as RoutingResult, UnnamedPeer, FIRST_SECTION_MAX_AGE,
     FIRST_SECTION_MIN_AGE, MIN_ADULT_AGE, MIN_AGE,
 };
+use crate::{elder_count, init_test_logger};
+
 use crate::types::{Keypair, PublicKey};
 use assert_matches::assert_matches;
 use bls_dkg::message::Message;
@@ -60,7 +62,7 @@ static TEST_EVENT_CHANNEL_SIZE: usize = 20;
 #[tokio::test(flavor = "multi_thread")]
 async fn receive_join_request_without_resource_proof_response() -> Result<()> {
     let prefix1 = Prefix::default().pushed(true);
-    let (section_auth, mut nodes, sk_set) = gen_section_authority_provider(prefix1, ELDER_SIZE);
+    let (section_auth, mut nodes, sk_set) = gen_section_authority_provider(prefix1, elder_count());
 
     let pk_set = sk_set.public_keys();
     let section_key = pk_set.public_key();
@@ -102,7 +104,7 @@ async fn receive_join_request_without_resource_proof_response() -> Result<()> {
 
     let mut commands = get_internal_commands(
         Command::HandleMessage {
-            sender_addr: new_node.addr,
+            sender: UnnamedPeer::addressed(new_node.addr),
             wire_msg,
             original_bytes: None,
         },
@@ -140,7 +142,7 @@ async fn receive_join_request_without_resource_proof_response() -> Result<()> {
 #[tokio::test(flavor = "multi_thread")]
 async fn receive_join_request_with_resource_proof_response() -> Result<()> {
     let prefix1 = Prefix::default().pushed(true);
-    let (section_auth, mut nodes, sk_set) = gen_section_authority_provider(prefix1, ELDER_SIZE);
+    let (section_auth, mut nodes, sk_set) = gen_section_authority_provider(prefix1, elder_count());
 
     let pk_set = sk_set.public_keys();
     let section_key = pk_set.public_key();
@@ -195,7 +197,7 @@ async fn receive_join_request_with_resource_proof_response() -> Result<()> {
 
     let commands = get_internal_commands(
         Command::HandleMessage {
-            sender_addr: new_node.addr,
+            sender: UnnamedPeer::addressed(new_node.addr),
             wire_msg,
             original_bytes: None,
         },
@@ -228,7 +230,7 @@ async fn receive_join_request_with_resource_proof_response() -> Result<()> {
 
 #[tokio::test(flavor = "multi_thread")]
 async fn receive_join_request_from_relocated_node() -> Result<()> {
-    crate::init_test_logger();
+    init_test_logger();
     let _span = tracing::info_span!("receive_join_request_from_relocated_node").entered();
 
     let (section_auth, mut nodes, sk_set) = create_section_auth();
@@ -304,7 +306,7 @@ async fn receive_join_request_from_relocated_node() -> Result<()> {
 
     let inner_commands = get_internal_commands(
         Command::HandleMessage {
-            sender_addr: relocated_node.addr,
+            sender: UnnamedPeer::addressed(relocated_node.addr),
             wire_msg,
             original_bytes: None,
         },
@@ -353,15 +355,16 @@ async fn aggregate_proposals() -> Result<()> {
     let dispatcher = Dispatcher::new(core);
 
     let new_peer = create_peer(MIN_AGE);
-    let node_state = NodeState::joined(&new_peer, None);
+    let node_state = NodeState::joined(new_peer.clone(), None);
     let proposal = Proposal::Online {
-        node_state: node_state.into_msg(),
+        node_state,
         dst_key: None,
     };
 
     let section_pk = section.section_key().await;
-    for (index, node) in nodes.iter().enumerate().take(THRESHOLD) {
-        let sig_share = proposal.prove(pk_set.clone(), index, &sk_set.secret_key_share(index))?;
+    for (index, node) in nodes.iter().enumerate().take(threshold()) {
+        let sig_share =
+            proposal.sign_with_key_share(pk_set.clone(), index, &sk_set.secret_key_share(index))?;
 
         let wire_msg = WireMsg::single_src(
             node,
@@ -370,7 +373,7 @@ async fn aggregate_proposals() -> Result<()> {
                 section_pk,
             },
             SystemMsg::Propose {
-                proposal: proposal.clone(),
+                proposal: proposal.clone().into_msg(),
                 sig_share,
             },
             section_auth.section_key(),
@@ -378,7 +381,7 @@ async fn aggregate_proposals() -> Result<()> {
 
         let commands = get_internal_commands(
             Command::HandleMessage {
-                sender_addr: node.addr,
+                sender: UnnamedPeer::addressed(node.addr),
                 wire_msg,
                 original_bytes: None,
             },
@@ -393,20 +396,20 @@ async fn aggregate_proposals() -> Result<()> {
         }
     }
 
-    let sig_share = proposal.prove(
+    let sig_share = proposal.sign_with_key_share(
         pk_set.clone(),
-        THRESHOLD,
-        &sk_set.secret_key_share(THRESHOLD),
+        threshold(),
+        &sk_set.secret_key_share(threshold()),
     )?;
     let section_pk = section.section_key().await;
     let wire_msg = WireMsg::single_src(
-        &nodes[THRESHOLD],
+        &nodes[threshold()],
         DstLocation::Section {
             name: XorName::from(PublicKey::Bls(section_pk)),
             section_pk,
         },
         SystemMsg::Propose {
-            proposal: proposal.clone(),
+            proposal: proposal.clone().into_msg(),
             sig_share,
         },
         section_auth.section_key(),
@@ -414,7 +417,7 @@ async fn aggregate_proposals() -> Result<()> {
 
     let mut commands = get_internal_commands(
         Command::HandleMessage {
-            sender_addr: nodes[THRESHOLD].addr,
+            sender: UnnamedPeer::addressed(nodes[threshold()].addr),
             wire_msg,
             original_bytes: None,
         },
@@ -446,7 +449,7 @@ async fn handle_agreement_on_online() -> Result<()> {
 
     let prefix = Prefix::default();
 
-    let (section_auth, mut nodes, sk_set) = gen_section_authority_provider(prefix, ELDER_SIZE);
+    let (section_auth, mut nodes, sk_set) = gen_section_authority_provider(prefix, elder_count());
     let (section, section_key_share) = create_section(&sk_set, &section_auth).await?;
     let node = nodes.remove(0);
     let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
@@ -482,20 +485,20 @@ async fn handle_agreement_on_online_of_elder_candidate() -> Result<()> {
     let chain = SecuredLinkedList::new(sk_set.secret_key().public_key());
 
     // Creates nodes where everybody has age 6 except one has 5.
-    let mut nodes: Vec<_> = gen_sorted_nodes(&Prefix::default(), ELDER_SIZE, true);
+    let mut nodes: Vec<_> = gen_sorted_nodes(&Prefix::default(), elder_count(), true);
 
     let section_auth = SectionAuthorityProvider::new(
         nodes.iter().map(Node::peer),
         Prefix::default(),
         sk_set.public_keys(),
     );
-    let section_signed_sap = section_signed(sk_set.secret_key(), section_auth.clone())?;
+    let signed_sap = section_signed(sk_set.secret_key(), section_auth.clone())?;
 
-    let section = NetworkKnowledge::new(*chain.root_key(), chain, section_signed_sap, None)?;
+    let section = NetworkKnowledge::new(*chain.root_key(), chain, signed_sap, None)?;
     let mut expected_new_elders = BTreeSet::new();
 
-    for peer in section_auth.peers() {
-        let node_state = NodeState::joined(&peer, None);
+    for peer in section_auth.elders() {
+        let node_state = NodeState::joined(peer.clone(), None);
         let sig = prove(sk_set.secret_key(), &node_state)?;
         let _updated = section
             .update_member(SectionAuth {
@@ -528,9 +531,9 @@ async fn handle_agreement_on_online_of_elder_candidate() -> Result<()> {
     // Handle agreement on Online of a peer that is older than the youngest
     // current elder - that means this peer is going to be promoted.
     let new_peer = create_peer(MIN_AGE + 2);
-    let node_state = NodeState::joined(&new_peer, Some(XorName::random()));
+    let node_state = NodeState::joined(new_peer.clone(), Some(XorName::random()));
     let proposal = Proposal::Online {
-        node_state: node_state.into_msg(),
+        node_state,
         dst_key: Some(sk_set.secret_key().public_key()),
     };
     let sig = keyed_signed(sk_set.secret_key(), &proposal.as_signable_bytes()?);
@@ -541,7 +544,7 @@ async fn handle_agreement_on_online_of_elder_candidate() -> Result<()> {
 
     // Verify we sent a `DkgStart` message with the expected participants.
     let mut dkg_start_sent = false;
-    let _changed = expected_new_elders.insert(new_peer);
+    let _changed = expected_new_elders.insert(&new_peer);
 
     for command in commands {
         let (recipients, wire_msg) = match command {
@@ -584,9 +587,9 @@ async fn handle_online_command(
     dispatcher: &Dispatcher,
     section_auth: &SectionAuthorityProvider,
 ) -> Result<HandleOnlineStatus> {
-    let node_state = NodeState::joined(peer, None);
+    let node_state = NodeState::joined(peer.clone(), None);
     let proposal = Proposal::Online {
-        node_state: node_state.into_msg(),
+        node_state,
         dst_key: None,
     };
     let sig = keyed_signed(sk_set.secret_key(), &proposal.as_signable_bytes()?);
@@ -616,11 +619,11 @@ async fn handle_online_command(
                 ..
             }) => {
                 if let JoinResponse::Approval {
-                    section_auth: section_signed_sap,
+                    section_auth: signed_sap,
                     ..
                 } = *response
                 {
-                    assert_eq!(section_signed_sap.value, section_auth.clone().into_msg());
+                    assert_eq!(signed_sap.value, section_auth.clone().to_msg());
                     assert_eq!(recipients, [peer]);
                     status.node_approval_sent = true;
                 }
@@ -659,12 +662,12 @@ async fn handle_agreement_on_online_of_rejoined_node(phase: NetworkPhase, age: u
         NetworkPhase::Startup => Prefix::default(),
         NetworkPhase::Regular => "0".parse().unwrap(),
     };
-    let (section_auth, mut nodes, sk_set) = gen_section_authority_provider(prefix, ELDER_SIZE);
+    let (section_auth, mut nodes, sk_set) = gen_section_authority_provider(prefix, elder_count());
     let (section, section_key_share) = create_section(&sk_set, &section_auth).await?;
 
     // Make a left peer.
     let peer = create_peer(age);
-    let node_state = NodeState::left(&peer, None);
+    let node_state = NodeState::left(peer.clone(), None);
     let node_state = section_signed(sk_set.secret_key(), node_state)?;
     let _updated = section.update_member(node_state).await;
 
@@ -726,17 +729,20 @@ async fn handle_agreement_on_online_of_rejoined_node_with_low_age_after_startup(
 
 #[tokio::test(flavor = "multi_thread")]
 async fn handle_agreement_on_offline_of_non_elder() -> Result<()> {
+    init_test_logger();
+    let _span = tracing::info_span!("handle_agreement_on_offline_of_non_elder").entered();
+
     let (section_auth, mut nodes, sk_set) = create_section_auth();
 
     let (section, section_key_share) = create_section(&sk_set, &section_auth).await?;
 
     let existing_peer = create_peer(MIN_AGE);
 
-    let node_state = NodeState::joined(&existing_peer, None);
+    let node_state = NodeState::joined(existing_peer.clone(), None);
     let node_state = section_signed(sk_set.secret_key(), node_state)?;
     let _updated = section.update_member(node_state).await;
 
-    let (event_tx, mut event_rx) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
+    let (event_tx, _event_rx) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
     let node = nodes.remove(0);
     let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
@@ -752,18 +758,20 @@ async fn handle_agreement_on_offline_of_non_elder() -> Result<()> {
     .await?;
     let dispatcher = Dispatcher::new(core);
 
-    let node_state = NodeState::left(&existing_peer, None);
-    let proposal = Proposal::Offline(node_state.into_msg());
+    let node_state = NodeState::left(existing_peer.clone(), None);
+    let proposal = Proposal::Offline(node_state);
     let sig = keyed_signed(sk_set.secret_key(), &proposal.as_signable_bytes()?);
 
     let _commands = dispatcher
         .handle_command(Command::HandleAgreement { proposal, sig }, "cmd-id")
         .await?;
 
-    assert_matches!(event_rx.recv().await, Some(Event::MemberLeft { name, age, }) => {
-        assert_eq!(name, existing_peer.name());
-        assert_eq!(age, MIN_AGE);
-    });
+    assert!(!dispatcher
+        .core
+        .network_knowledge()
+        .members()
+        .all_members()
+        .contains(&existing_peer));
 
     Ok(())
 }
@@ -775,12 +783,12 @@ async fn handle_agreement_on_offline_of_elder() -> Result<()> {
     let (section, section_key_share) = create_section(&sk_set, &section_auth).await?;
 
     let existing_peer = create_peer(MIN_AGE);
-    let node_state = NodeState::joined(&existing_peer, None);
+    let node_state = NodeState::joined(existing_peer.clone(), None);
     let node_state = section_signed(sk_set.secret_key(), node_state)?;
     let _updated = section.update_member(node_state).await;
 
     // Pick the elder to remove.
-    let auth_peers = section_auth.peers();
+    let auth_peers = section_auth.elders();
     let remove_peer = auth_peers.last().expect("section_auth is empty");
 
     let remove_node_state = section
@@ -790,7 +798,7 @@ async fn handle_agreement_on_offline_of_elder() -> Result<()> {
         .leave()?;
 
     // Create our node
-    let (event_tx, mut event_rx) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
+    let (event_tx, _event_rx) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
     let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let node = nodes.remove(0);
     let node_name = node.name();
@@ -808,7 +816,7 @@ async fn handle_agreement_on_offline_of_elder() -> Result<()> {
     let dispatcher = Dispatcher::new(core);
 
     // Handle agreement on the Offline proposal
-    let proposal = Proposal::Offline(remove_node_state.into_msg());
+    let proposal = Proposal::Offline(remove_node_state);
     let sig = keyed_signed(sk_set.secret_key(), &proposal.as_signable_bytes()?);
 
     let commands = dispatcher
@@ -837,10 +845,10 @@ async fn handle_agreement_on_offline_of_elder() -> Result<()> {
         };
 
         let expected_new_elders: BTreeSet<_> = section_auth
-            .peers()
-            .into_iter()
-            .filter(|peer| peer != remove_peer)
-            .chain(iter::once(existing_peer.clone()))
+            .elders()
+            .filter(|peer| peer != &remove_peer)
+            .chain(iter::once(&existing_peer))
+            .cloned()
             .collect();
         itertools::assert_equal(actual_elder_candidates, expected_new_elders.clone());
 
@@ -856,9 +864,12 @@ async fn handle_agreement_on_offline_of_elder() -> Result<()> {
 
     assert!(dkg_start_sent);
 
-    assert_matches!(event_rx.recv().await, Some(Event::MemberLeft { name, .. }) => {
-        assert_eq!(name, remove_peer.name());
-    });
+    assert!(!dispatcher
+        .core
+        .network_knowledge()
+        .members()
+        .all_members()
+        .contains(remove_peer));
 
     // The removed peer is still our elder because we haven't yet processed the section update.
     assert!(dispatcher
@@ -880,6 +891,9 @@ enum UntrustedMessageSource {
 #[tokio::test(flavor = "multi_thread")]
 // Checking when we get AE info that is ahead of us we should handle it.
 async fn ae_msg_from_the_future_is_handled() -> Result<()> {
+    init_test_logger();
+    let _span = info_span!("ae_msg_from_the_future_is_handled").entered();
+
     // Create first `Section` with a chain of length 2
     let sk0 = bls::SecretKey::random();
     let pk0 = sk0.public_key();
@@ -924,9 +938,9 @@ async fn ae_msg_from_the_future_is_handled() -> Result<()> {
     // Create the new `SectionAuthorityProvider` by replacing the last peer with a new one.
     let new_peer = create_peer(MIN_AGE);
     let new_elders = old_sap
-        .peers()
-        .into_iter()
+        .elders()
         .take(old_sap.elder_count() - 1)
+        .cloned()
         .chain(vec![new_peer]);
 
     let new_sap =
@@ -942,7 +956,7 @@ async fn ae_msg_from_the_future_is_handled() -> Result<()> {
             section_pk: pk1,
         },
         SystemMsg::AntiEntropyUpdate {
-            section_auth: new_sap.into_msg(),
+            section_auth: new_sap.to_msg(),
             members: None,
             section_signed: signed_new_sap.sig,
             proof_chain: chain,
@@ -960,7 +974,7 @@ async fn ae_msg_from_the_future_is_handled() -> Result<()> {
 
     let _commands = get_internal_commands(
         Command::HandleMessage {
-            sender_addr: old_node.addr,
+            sender: UnnamedPeer::addressed(old_node.addr),
             wire_msg,
             original_bytes: None,
         },
@@ -985,7 +999,7 @@ async fn ae_msg_from_the_future_is_handled() -> Result<()> {
 #[tokio::test(flavor = "multi_thread")]
 // Checking when we send AE info to a section from untrusted section, we do not handle it and error out
 async fn untrusted_ae_message_msg_errors() -> Result<()> {
-    crate::init_test_logger();
+    init_test_logger();
     let _span = tracing::info_span!("untrusted_ae_message_msg_errors").entered();
 
     let (our_section_auth, _, sk_set0) = create_section_auth();
@@ -1003,7 +1017,7 @@ async fn untrusted_ae_message_msg_errors() -> Result<()> {
     // a valid AE msg but with a non-verifiable SAP...
     let bogus_section_pk = bls::SecretKey::random().public_key();
     let node_msg = SystemMsg::AntiEntropyUpdate {
-        section_auth: section_signed_our_section_auth.value.clone().into_msg(),
+        section_auth: section_signed_our_section_auth.value.clone().to_msg(),
         section_signed: section_signed_our_section_auth.sig,
         proof_chain: SecuredLinkedList::new(bogus_section_pk),
         members: None,
@@ -1040,7 +1054,7 @@ async fn untrusted_ae_message_msg_errors() -> Result<()> {
 
     let _commands = get_internal_commands(
         Command::HandleMessage {
-            sender_addr: sender.addr,
+            sender: UnnamedPeer::addressed(sender.addr),
             wire_msg,
             original_bytes: None,
         },
@@ -1086,7 +1100,9 @@ async fn relocation_of_non_elder() -> Result<()> {
     relocation(RelocatedPeerRole::NonElder).await
 }
 
-const THRESHOLD: usize = supermajority(ELDER_SIZE) - 1;
+fn threshold() -> usize {
+    supermajority(elder_count()) - 1
+}
 
 #[allow(dead_code)]
 enum RelocatedPeerRole {
@@ -1096,11 +1112,11 @@ enum RelocatedPeerRole {
 
 async fn relocation(relocated_peer_role: RelocatedPeerRole) -> Result<()> {
     let prefix: Prefix = "0".parse().unwrap();
-    let (section_auth, mut nodes, sk_set) = gen_section_authority_provider(prefix, ELDER_SIZE);
+    let (section_auth, mut nodes, sk_set) = gen_section_authority_provider(prefix, elder_count());
     let (section, section_key_share) = create_section(&sk_set, &section_auth).await?;
 
     let non_elder_peer = create_peer(MIN_AGE);
-    let node_state = NodeState::joined(&non_elder_peer, None);
+    let node_state = NodeState::joined(non_elder_peer.clone(), None);
     let node_state = section_signed(sk_set.secret_key(), node_state)?;
     assert!(section.update_member(node_state).await);
     let node = nodes.remove(0);
@@ -1120,10 +1136,10 @@ async fn relocation(relocated_peer_role: RelocatedPeerRole) -> Result<()> {
 
     let relocated_peer = match relocated_peer_role {
         RelocatedPeerRole::Elder => section_auth
-            .peers()
-            .into_iter()
+            .elders()
             .nth(1)
-            .expect("too few elders"),
+            .expect("too few elders")
+            .clone(),
         RelocatedPeerRole::NonElder => non_elder_peer,
     };
 
@@ -1252,7 +1268,7 @@ async fn message_to_self(dst: MessageDst) -> Result<()> {
     assert!(commands.is_empty());
 
     let msg_type = assert_matches!(comm_rx.recv().await, Some(ConnectionEvent::Received((sender, bytes))) => {
-        assert_eq!(sender, node.addr);
+        assert_eq!(sender.addr(), node.addr);
         assert_matches!(WireMsg::deserialize(bytes), Ok(msg_type) => msg_type)
     });
 
@@ -1269,13 +1285,13 @@ async fn message_to_self(dst: MessageDst) -> Result<()> {
 
 #[tokio::test(flavor = "multi_thread")]
 async fn handle_elders_update() -> Result<()> {
-    crate::init_test_logger();
+    init_test_logger();
     let _span = tracing::info_span!("handle_elders_update").entered();
-    // Start with section that has `ELDER_SIZE` elders with age 6, 1 non-elder with age 5 and one
+    // Start with section that has `elder_count()` elders with age 6, 1 non-elder with age 5 and one
     // to-be-elder with age 7:
     let node = create_node(MIN_AGE + 2, None);
     let mut other_elder_peers: Vec<_> = iter::repeat_with(|| create_peer(MIN_AGE + 2))
-        .take(ELDER_SIZE - 1)
+        .take(elder_count() - 1)
         .collect();
     let adult_peer = create_peer(MIN_ADULT_AGE);
     let promoted_peer = create_peer(MIN_AGE + 3);
@@ -1292,7 +1308,7 @@ async fn handle_elders_update() -> Result<()> {
     let (section0, section_key_share) = create_section(&sk_set0, &sap0).await?;
 
     for peer in [&adult_peer, &promoted_peer] {
-        let node_state = NodeState::joined(peer, None);
+        let node_state = NodeState::joined(peer.clone(), None);
         let node_state = section_signed(sk_set0.secret_key(), node_state)?;
         assert!(section0.update_member(node_state).await);
     }
@@ -1314,7 +1330,7 @@ async fn handle_elders_update() -> Result<()> {
     let elder_names1: BTreeSet<_> = sap1.names();
 
     let signed_sap1 = section_signed(sk_set1.secret_key(), sap1)?;
-    let proposal = Proposal::OurElders(signed_sap1.into_authed_msg());
+    let proposal = Proposal::OurElders(signed_sap1);
     let signature = sk_set0.secret_key().sign(&proposal.as_signable_bytes()?);
     let sig = KeyedSig {
         signature,
@@ -1404,22 +1420,21 @@ async fn handle_elders_update() -> Result<()> {
 // Test that demoted node still sends `Sync` messages on split.
 #[tokio::test(flavor = "multi_thread")]
 async fn handle_demote_during_split() -> Result<()> {
-    crate::init_test_logger();
+    init_test_logger();
     let _span = tracing::info_span!("handle_demote_during_split").entered();
 
     let node = create_node(MIN_ADULT_AGE, None);
     let node_name = node.name();
     let prefix0 = Prefix::default().pushed(false);
     let prefix1 = Prefix::default().pushed(true);
-
     // These peers together with `node` are pre-split elders.
     // These peers together with `peer_c` are prefix-0 post-split elders.
     let peers_a: Vec<_> = iter::repeat_with(|| create_peer_in_prefix(&prefix0, MIN_ADULT_AGE))
-        .take(ELDER_SIZE - 1)
+        .take(elder_count() - 1)
         .collect();
     // These peers are prefix-1 post-split elders.
     let peers_b: Vec<_> = iter::repeat_with(|| create_peer_in_prefix(&prefix1, MIN_ADULT_AGE))
-        .take(ELDER_SIZE)
+        .take(elder_count())
         .collect();
     // This peer is a prefix-0 post-split elder.
     let peer_c = create_peer_in_prefix(&prefix0, MIN_ADULT_AGE);
@@ -1431,10 +1446,9 @@ async fn handle_demote_during_split() -> Result<()> {
         Prefix::default(),
         sk_set_v0.public_keys(),
     );
-
     let (section, section_key_share) = create_section(&sk_set_v0, &section_auth_v0).await?;
 
-    for peer in peers_b.iter().chain(iter::once(&peer_c)) {
+    for peer in peers_b.iter().chain(iter::once(&peer_c)).cloned() {
         let node_state = NodeState::joined(peer, None);
         let node_state = section_signed(sk_set_v0.secret_key(), node_state)?;
         assert!(section.update_member(node_state).await);
@@ -1472,8 +1486,8 @@ async fn handle_demote_during_split() -> Result<()> {
     let dispatcher = Dispatcher::new(core);
 
     // Create agreement on `OurElder` for both sub-sections
-    let create_our_elders_command = |section_signed_sap| -> Result<_> {
-        let proposal = Proposal::OurElders(section_signed_sap);
+    let create_our_elders_command = |signed_sap| -> Result<_> {
+        let proposal = Proposal::OurElders(signed_sap);
         let signature = sk_set_v0.secret_key().sign(&proposal.as_signable_bytes()?);
         let sig = KeyedSig {
             signature,
@@ -1490,8 +1504,8 @@ async fn handle_demote_during_split() -> Result<()> {
         sk_set_v1_p0.public_keys(),
     );
 
-    let section_signed_sap = section_signed(sk_set_v1_p0.secret_key(), section_auth)?;
-    let command = create_our_elders_command(section_signed_sap.into_authed_msg())?;
+    let signed_sap = section_signed(sk_set_v1_p0.secret_key(), section_auth)?;
+    let command = create_our_elders_command(signed_sap)?;
     let commands = dispatcher.handle_command(command, "cmd-id-1").await?;
 
     assert_matches!(&commands[..], &[]);
@@ -1500,8 +1514,8 @@ async fn handle_demote_during_split() -> Result<()> {
     let section_auth =
         SectionAuthorityProvider::new(peers_b.iter().cloned(), prefix1, sk_set_v1_p1.public_keys());
 
-    let section_signed_sap = section_signed(sk_set_v1_p1.secret_key(), section_auth)?;
-    let command = create_our_elders_command(section_signed_sap.into_authed_msg())?;
+    let signed_sap = section_signed(sk_set_v1_p1.secret_key(), section_auth)?;
+    let command = create_our_elders_command(signed_sap)?;
 
     let commands = dispatcher.handle_command(command, "cmd-id-2").await?;
 
@@ -1530,13 +1544,8 @@ async fn handle_demote_during_split() -> Result<()> {
         }
     }
 
-    if prefix0.matches(&node_name) {
-        // everyone
-        assert_eq!(update_recipients.len(), 14);
-    } else {
-        // our node's whole section plus peer c
-        assert_eq!(update_recipients.len(), 8);
-    }
+    // our node's whole section
+    assert_eq!(update_recipients.len(), elder_count());
 
     Ok(())
 }
@@ -1566,7 +1575,7 @@ pub(crate) async fn create_comm() -> Result<Comm> {
 // Generate random SectionAuthorityProvider and the corresponding Nodes.
 fn create_section_auth() -> (SectionAuthorityProvider, Vec<Node>, SecretKeySet) {
     let (section_auth, elders, secret_key_set) =
-        gen_section_authority_provider(Prefix::default(), ELDER_SIZE);
+        gen_section_authority_provider(Prefix::default(), elder_count());
     (section_auth, elders, secret_key_set)
 }
 
@@ -1583,17 +1592,13 @@ async fn create_section(
     section_auth: &SectionAuthorityProvider,
 ) -> Result<(NetworkKnowledge, SectionKeyShare)> {
     let section_chain = SecuredLinkedList::new(sk_set.public_keys().public_key());
-    let section_signed_sap = section_signed(sk_set.secret_key(), section_auth.clone())?;
+    let signed_sap = section_signed(sk_set.secret_key(), section_auth.clone())?;
 
-    let section = NetworkKnowledge::new(
-        *section_chain.root_key(),
-        section_chain,
-        section_signed_sap,
-        None,
-    )?;
+    let section =
+        NetworkKnowledge::new(*section_chain.root_key(), section_chain, signed_sap, None)?;
 
-    for peer in section_auth.peers() {
-        let node_state = NodeState::joined(&peer, None);
+    for peer in section_auth.elders() {
+        let node_state = NodeState::joined(peer.clone(), None);
         let node_state = section_signed(sk_set.secret_key(), node_state)?;
         let _updated = section.update_member(node_state).await;
     }
@@ -1611,8 +1616,7 @@ async fn create_section(
 fn create_relocation_trigger(sk: &bls::SecretKey, age: u8) -> Result<(Proposal, KeyedSig)> {
     loop {
         let proposal = Proposal::Online {
-            node_state: NodeState::joined(&create_peer(MIN_ADULT_AGE), Some(rand::random()))
-                .into_msg(),
+            node_state: NodeState::joined(create_peer(MIN_ADULT_AGE), Some(rand::random())),
             dst_key: None,
         };
 
@@ -1639,7 +1643,7 @@ pub(crate) struct SecretKeySet {
 
 impl SecretKeySet {
     pub(crate) fn random() -> Self {
-        let poly = bls::poly::Poly::random(THRESHOLD, &mut rand::thread_rng());
+        let poly = bls::poly::Poly::random(threshold(), &mut rand::thread_rng());
         let key = bls::SecretKey::from_mut(&mut poly.evaluate(0));
         let set = bls::SecretKeySet::from(poly);
 
